@@ -38,6 +38,24 @@ using std::set;
 #include "conftree.h"
 #include "smallut.h"
 
+class RclConfig;
+
+// A small class used for parameters that need to be computed from the
+// config string, and which can change with the keydir. Minimize work
+// by using the keydirgen and a saved string to avoid unneeded
+// recomputations
+class ParamStale {
+public:
+    RclConfig *parent;
+    ConfNull  *conffile;
+    string    paramname;
+    int       savedkeydirgen;
+    string    savedvalue;
+
+    void init(RclConfig *rconf, ConfNull *cnf, const string& nm);
+    bool needrecompute();
+};
+
 class RclConfig {
  public:
 
@@ -83,6 +101,21 @@ class RclConfig {
     /** Variant with autoconversion to bool */
     bool getConfParam(const std::string &name, bool *value);
 
+    /** 
+     * Get list of config names under current sk, with possible 
+     * wildcard filtering 
+     */
+    list<string> getConfNames(const char *pattern = 0) {
+	return m_conf->getNames(m_keydir, pattern);
+    }
+
+    /** Check if name exists anywhere in config */
+    bool hasNameAnywhere(const string& nm) 
+    {
+        return m_conf? m_conf->hasNameAnywhere(nm) : false;
+    }
+
+
     /** Get default charset for current keydir (was set during setKeydir) 
      * filenames are handled differently */
     const string &getDefCharset(bool filename = false);
@@ -100,11 +133,10 @@ class RclConfig {
     string getStopfile();
 
     /** Get list of skipped file names for current keydir */
-    list<string> getSkippedNames();
+    list<string>& getSkippedNames();
 
     /** Get list of skipped paths patterns. Doesn't depend on the keydir */
     list<string> getSkippedPaths();
-
     /** Get list of skipped paths patterns, daemon version (may add some)
 	Doesn't depend on the keydir */
     list<string> getDaemSkippedPaths();
@@ -135,6 +167,10 @@ class RclConfig {
 
     /** mimeconf: get input filter for mimetype */
     string getMimeHandlerDef(const string &mimetype, bool filtertypes=false);
+    /** For lines like: "name = some value; and additional; attributes"
+     * Split the value and store the attributes in a ConfSimple */
+    bool valueSplitAttributes(const string& whole, string& value, 
+                              ConfSimple& attrs);
 
     /** mimeconf: get icon name for mimetype */
     string getMimeIconName(const string &mtype, string *path = 0);
@@ -161,7 +197,7 @@ class RclConfig {
     const map<string, string>& getXattrToField() {return m_xattrtofld;}
     
     /** mimeview: get/set external viewer exec string(s) for mimetype(s) */
-    string getMimeViewerDef(const string &mimetype);
+    string getMimeViewerDef(const string &mimetype, const string& apptag);
     bool getMimeViewerDefs(vector<pair<string, string> >&);
     bool setMimeViewerDef(const string& mimetype, const string& cmd);
 
@@ -187,9 +223,8 @@ class RclConfig {
 	}
 	return *this;
     }
-    list<string> getConfNames(const string &sk) {
-	return m_conf->getNames(sk);
-    }
+
+    friend class ParamStale;
 
  private:
     int m_ok;
@@ -197,6 +232,8 @@ class RclConfig {
     string m_confdir;   // User directory where the customized files are stored
     string m_datadir;   // Example: /usr/local/share/recoll
     string m_keydir;    // Current directory used for parameter fetches.
+    int    m_keydirgen; // To help with knowing when to update computed data.
+
     list<string> m_cdirs; // directory stack for the confstacks
 
     ConfStack<ConfTree> *m_conf;   // Parsed configuration files
@@ -211,12 +248,16 @@ class RclConfig {
 
     void        *m_stopsuffixes;
     unsigned int m_maxsufflen;
+    ParamStale   m_stpsuffstate;
+
+    ParamStale   m_skpnstate;
+    list<string> m_skpnlist;
 
     // Parameters auto-fetched on setkeydir
     string defcharset;   // These are stored locally to avoid 
     bool   guesscharset; // They are fetched initially or on setKeydir()
     // Limiting set of mime types to be processed. Normally empty.
-    string        m_rmtstr;
+    ParamStale    m_rmtstate;
     set<string>   m_restrictMTypes; 
 
     /** Create initial user configuration */
@@ -224,16 +265,7 @@ class RclConfig {
     /** Copy from other */
     void initFrom(const RclConfig& r);
     /** Init pointers to 0 */
-    void zeroMe() {
-	m_ok = false; 
-	m_conf = 0; 
-	mimemap = 0; 
-	mimeconf = 0; 
-	mimeview = 0; 
-	m_fields = 0;
-	m_stopsuffixes = 0;
-	m_maxsufflen = 0;
-    }
+    void zeroMe();
     /** Free data then zero pointers */
     void freeAll();
     bool readFieldsConfig(const string& errloc);
