@@ -23,7 +23,10 @@ static char rcsid[] = "@(#$Id: recollindex.cpp,v 1.38 2008-10-14 06:07:42 dockes
 
 #include <stdio.h>
 #include <signal.h>
+#include <errno.h>
 #include <sys/stat.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 
 #include <iostream>
 #include <list>
@@ -39,13 +42,25 @@ using namespace std;
 #include "pathut.h"
 #include "rclmon.h"
 #include "x11mon.h"
-#include "rclversion.h"
+#include "cancelcheck.h"
+#include "rcldb.h"
+#include "beaglequeue.h"
+#include "recollindex.h"
+#include "fsindexer.h"
 
-// Globals for exit cleanup
-ConfIndexer *confindexer;
-DbIndexer *dbindexer;
+// Globals for atexit cleanup
+static ConfIndexer *confindexer;
 
+// This is set as an atexit routine, 
+static void cleanup()
+{
+    deleteZ(confindexer);
+}
+
+// Global stop request flag. This is checked in a number of place in the
+// indexing routines.
 int stopindexing;
+
 // Mainly used to request indexing stop, we currently do not use the
 // current file name
 class MyUpdater : public DbIxStatusUpdater {
@@ -57,132 +72,63 @@ class MyUpdater : public DbIxStatusUpdater {
 	return true;
     }
 };
-MyUpdater updater;
+static MyUpdater updater;
 
 static void sigcleanup(int sig)
 {
-    fprintf(stderr, "sigcleanup\n");
-    LOGDEB(("sigcleanup\n"));
+    fprintf(stderr, "Got signal, registering stop request\n");
+    LOGDEB(("Got signal, registering stop request\n"));
+    CancelCheck::instance().setCancel();
     stopindexing = 1;
 }
 
-static bool makeDbIndexer(RclConfig *config)
+static bool makeIndexer(RclConfig *config)
 {
-    string dbdir = config->getDbDir();
-    if (dbdir.empty()) {
-	fprintf(stderr, "makeDbIndexer: no database directory in "
-		"configuration for %s\n", config->getKeyDir().c_str());
-	return false;
+    if (!confindexer)
+	confindexer = new ConfIndexer(config, &updater);
+    if (!confindexer) {
+        cerr << "Cannot create indexer" << endl;
+        exit(1);
     }
-    // Check if there is already an indexer for the right db
-    if (dbindexer && dbindexer->getDbDir().compare(dbdir)) {
-	deleteZ(dbindexer);
-    }
-
-    if (!dbindexer)
-	dbindexer = new DbIndexer(config, dbdir, &updater);
-
     return true;
 }
 
-// The list of top directories/files wont change during program run,
-// let's cache it:
-static list<string> o_tdl;
-
-// Index a list of files. We just check that they belong to one of the topdirs
-// subtrees, and call the indexer method
-bool indexfiles(RclConfig *config, const list<string> &filenames)
+// Index a list of files. We just check that they belong to one of the
+// topdirs subtrees, and call the indexer method. 
+//
+// This is called either from the command line or from the monitor. In
+// this case we're called repeatedly in the same process, and the
+// confindexer is only created once by makeIndexer (but the db closed and
+// flushed every time)
+bool indexfiles(RclConfig *config, list<string> &filenames)
 {
     if (filenames.empty())
 	return true;
-    
-    if (o_tdl.empty()) {
-	o_tdl = config->getTopdirs();
-	if (o_tdl.empty()) {
-	    fprintf(stderr, "Top directory list (topdirs param.) "
-		    "not found in config or Directory list parse error");
-	    return false;
-	}
-    }
-
-    list<string> myfiles;
-    for (list<string>::const_iterator it = filenames.begin(); 
-	 it != filenames.end(); it++) {
-	string fn = path_canon(*it);
-	bool ok = false;
-	// Check that this file name belongs to one of our subtrees
-	for (list<string>::iterator dit = o_tdl.begin(); 
-	     dit != o_tdl.end(); dit++) {
-	    if (fn.find(*dit) == 0) {
-		myfiles.push_back(fn);
-		ok = true;
-		break;
-	    }
-	}
-	if (!ok) {
-	    fprintf(stderr, "File %s not in indexed area\n", fn.c_str());
-	}
-    }
-    if (myfiles.empty())
-	return true;
-
-    // Note: we should sort the file names against the topdirs here
-    // and check for different databases. But we can for now only have
-    // one database per config, so we set the keydir from the first
-    // file (which is not really needed...), create the indexer/db and
-    // go:
-    config->setKeyDir(path_getfather(*myfiles.begin()));
-
-    if (!makeDbIndexer(config) || !dbindexer)
+    if (!makeIndexer(config))
 	return false;
-    else
-	return dbindexer->indexFiles(myfiles);
+    return confindexer->indexFiles(filenames);
 }
 
-// Delete a list of files.
-bool purgefiles(RclConfig *config, const list<string> &filenames)
+// Delete a list of files. Same comments about call contexts as indexfiles.
+bool purgefiles(RclConfig *config, list<string> &filenames)
 {
     if (filenames.empty())
 	return true;
-    
-    if (o_tdl.empty()) {
-	o_tdl = config->getTopdirs();
-	if (o_tdl.empty()) {
-	    fprintf(stderr, "Top directory list (topdirs param.) "
-		    "not found in config or Directory list parse error");
-	    return false;
-	}
-    }
-
-    list<string> myfiles;
-    for (list<string>::const_iterator it = filenames.begin(); 
-	 it != filenames.end(); it++) {
-	myfiles.push_back(path_canon(*it));
-    }
-
-    // Note: we should sort the file names against the topdirs here
-    // and check for different databases. But we can for now only have
-    // one database per config, so we set the keydir from the first
-    // file (which is not really needed...), create the indexer/db and
-    // go:
-    config->setKeyDir(path_getfather(*myfiles.begin()));
-
-    if (!makeDbIndexer(config) || !dbindexer)
+    if (!makeIndexer(config))
 	return false;
-    else
-	return dbindexer->purgeFiles(myfiles);
+    return confindexer->purgeFiles(filenames);
 }
 
 // Create stemming and spelling databases
 bool createAuxDbs(RclConfig *config)
 {
-    if (!makeDbIndexer(config) || !dbindexer)
+    if (!makeIndexer(config))
 	return false;
 
-    if (!dbindexer->createStemmingDatabases())
+    if (!confindexer->createStemmingDatabases())
 	return false;
 
-    if (!dbindexer->createAspellDict())
+    if (!confindexer->createAspellDict())
 	return false;
 
     return true;
@@ -191,17 +137,9 @@ bool createAuxDbs(RclConfig *config)
 // Create additional stem database 
 static bool createstemdb(RclConfig *config, const string &lang)
 {
-    makeDbIndexer(config);
-    if (dbindexer)
-	return dbindexer->createStemDb(lang);
-    else
-	return false;
-}
-
-static void cleanup()
-{
-    deleteZ(confindexer);
-    deleteZ(dbindexer);
+    if (!makeIndexer(config))
+        return false;
+    return confindexer->createStemDb(lang);
 }
 
 static const char *thisprog;
@@ -219,6 +157,7 @@ static int     op_flags;
 #define OPT_w     0x400
 #define OPT_x     0x800
 #define OPT_l     0x1000
+#define OPT_b     0x2000
 
 static const char usage [] =
 "\n"
@@ -241,6 +180,8 @@ static const char usage [] =
 "    List available stemming languages\n"
 "recollindex -s <lang>\n"
 "    Build stem database for additional language <lang>\n"
+"recollindex -b\n"
+"    Process the Beagle queue\n"
 #ifdef RCL_USE_ASPELL
 "recollindex -S\n"
 "    Build aspell spelling dictionary.>\n"
@@ -254,7 +195,7 @@ Usage(FILE *where = stderr)
 {
     FILE *fp = (op_flags & OPT_h) ? stdout : stderr;
     fprintf(fp, "%s: Usage: %s", thisprog, usage);
-    fprintf(fp, "Recoll version: %s\n", rclversion);
+    fprintf(fp, "Recoll version: %s\n", Rcl::version_string().c_str());
     exit((op_flags & OPT_h)==0);
 }
 
@@ -278,6 +219,7 @@ int main(int argc, const char **argv)
 	    Usage();
 	while (**argv)
 	    switch (*(*argv)++) {
+	    case 'b': op_flags |= OPT_b; break;
 	    case 'c':	op_flags |= OPT_c; if (argc < 2)  Usage();
 		a_config = *(++argv);
 		argc--; goto b1;
@@ -325,6 +267,10 @@ int main(int argc, const char **argv)
 	exit(1);
     }
     bool rezero(op_flags & OPT_z);
+
+    if (setpriority(PRIO_PGRP, 0, 20) != 0) {
+        LOGINFO(("recollindex: can't setpriority(), errno %d\n", errno));
+    }
     
     if (op_flags & (OPT_i|OPT_e)) {
 	list<string> filenames;
@@ -342,16 +288,18 @@ int main(int argc, const char **argv)
 		filenames.push_back(*argv++);
 	    }
 	}
-
+        bool status;
 	if (op_flags & OPT_i)
-	    exit(!indexfiles(config, filenames));
+	    status = indexfiles(config, filenames);
 	else 
-	    exit(!purgefiles(config, filenames));
-
+	    status = purgefiles(config, filenames);
+        if (!confindexer->getReason().empty())
+            cerr << confindexer->getReason() << endl;
+        exit(status ? 0 : 1);
     } else if (op_flags & OPT_l) {
 	if (argc != 0) 
 	    Usage();
-	list<string> stemmers = DbIndexer::getStemmerNames();
+	list<string> stemmers = ConfIndexer::getStemmerNames();
 	for (list<string>::const_iterator it = stemmers.begin(); 
 	     it != stemmers.end(); it++) {
 	    cout << *it << endl;
@@ -371,6 +319,11 @@ int main(int argc, const char **argv)
 	    LOGDEB(("recollindex: daemonizing\n"));
 	    daemon(0,0);
 	}
+        // Not too sure if I have to redo the nice thing after daemon(),
+        // can't hurt anyway (easier than testing on all platforms...)
+        if (setpriority(PRIO_PGRP, 0, 20) != 0) {
+            LOGINFO(("recollindex: can't setpriority(), errno %d\n", errno));
+        }
 	if (sleepsecs > 0) {
 	    LOGDEB(("recollindex: sleeping %d\n", sleepsecs));
 	    sleep(sleepsecs);
@@ -380,7 +333,7 @@ int main(int argc, const char **argv)
 	    exit(0);
 
 	confindexer = new ConfIndexer(config, &updater);
-	confindexer->index(rezero);
+	confindexer->index(rezero, ConfIndexer::IxTAll);
 	deleteZ(confindexer);
 	int opts = RCLMON_NONE;
 	if (op_flags & OPT_D)
@@ -394,18 +347,20 @@ int main(int argc, const char **argv)
 
 #ifdef RCL_USE_ASPELL
     } else if (op_flags & OPT_S) {
-	makeDbIndexer(config);
-	if (dbindexer)
-	    exit(!dbindexer->createAspellDict());
-	else
-	    exit(1);
+	if (!makeIndexer(config))
+            exit(1);
+        exit(!confindexer->createAspellDict());
 #endif // ASPELL
-
+    } else if (op_flags & OPT_b) {
+        cerr << "Not yet" << endl;
+        return 1;
     } else {
 	confindexer = new ConfIndexer(config, &updater);
-	bool status = confindexer->index(rezero);
+	bool status = confindexer->index(rezero, ConfIndexer::IxTAll);
 	if (!status) 
 	    cerr << "Indexing failed" << endl;
-	exit(!status);
+        if (!confindexer->getReason().empty())
+            cerr << confindexer->getReason() << endl;
+	return !status;
     }
 }
