@@ -29,136 +29,98 @@ using namespace std;
 #include "debuglog.h"
 #include "rclconfig.h"
 #include "smallut.h"
-#ifdef RCL_USE_XATTR
-#include "pxattr.h"
-#endif // RCL_USE_XATTR
 
 #include "mh_exec.h"
+#include "mh_execm.h"
 #include "mh_html.h"
 #include "mh_mail.h"
 #include "mh_mbox.h"
 #include "mh_text.h"
 #include "mh_unknown.h"
 
-// Common code for all docs that are a file (not subdocs). If extended
-// attributes support is enabled, fetch the data.
-bool RecollFilter::set_document_file(const string& path)
-{
-#ifdef RCL_USE_XATTR
-    RclConfig* rclconfig = RclConfig::getMainConfig();
-    if (rclconfig == 0) {
-	LOGERR(("RecollFilter::set_document_file: no config\n"));
-	return false;
-    }
-    vector<string> xnames;
-    if (!pxattr::list(path, &xnames)) {
-	LOGERR(("xattrToMeta: pxattr::list failed, errno %d\n", errno));
-	return false;
-    }
-    const map<string, string>& xtof = rclconfig->getXattrToField();
-    for (vector<string>::const_iterator it = xnames.begin();
-	 it != xnames.end(); it++) {
-	map<string, string>::const_iterator mit;
-	if ((mit = xtof.find(*it)) != xtof.end()) {
-	    string value;
-	    if (!pxattr::get(path, *it, &value, pxattr::PXATTR_NOFOLLOW)) {
-		LOGERR(("xattrToMeta: pxattr::get failed for %s, errno %d\n", 
-			(*it).c_str(), errno));
-		continue;
-	    }
-	    // Encode should we ?
-	    m_fieldsFromAttrs[mit->second] = value;
-	}
-    }
-#endif // RCL_USE_XATTR
-    return true;
-}
-
-// Pool of already known and created handlers. There can be several instance
-// for a given mime type (think email attachment in email message)
+// Performance help: we use a pool of already known and created
+// handlers. There can be several instances for a given mime type
+// (think email attachment in email message: 2 rfc822 handlers are
+// needed simulteanously)
 static multimap<string, Dijon::Filter*>  o_handlers;
 
-/** Create internal handler object appropriate for given mime type */
+/** For mime types set as "internal" in mimeconf: 
+  * create appropriate handler object. */
 static Dijon::Filter *mhFactory(const string &mime)
 {
     string lmime(mime);
     stringtolower(lmime);
-    if ("text/plain" == lmime)
+    if ("text/plain" == lmime) {
 	return new MimeHandlerText(lmime);
-    else if ("text/html" == lmime)
+    } else if ("text/html" == lmime) {
 	return new MimeHandlerHtml(lmime);
-    else if ("text/x-mail" == lmime)
+    } else if ("text/x-mail" == lmime) {
 	return new MimeHandlerMbox(lmime);
-    else if ("message/rfc822" == lmime)
+    } else if ("message/rfc822" == lmime) {
 	return new MimeHandlerMail(lmime);
-    else 
+    } else if (lmime.find("text/") == 0) {
+        // Try to handle unknown text/xx as text/plain. This
+        // only happen if the text/xx was defined as "internal" in
+        // mimeconf, not at random. For programs, for example this
+        // allows indexing and previewing as text/plain (no filter
+        // exec) but still opening with a specific editor.
+        return new MimeHandlerText(lmime); 
+    } else {
 	return new MimeHandlerUnknown(lmime);
+    }
 }
 
 /**
  * Create a filter that executes an external program or script
- * A filter def can look like. 
- * exec someprog -v -t " h i j";charset= xx; mimetype=yy
- * We don't support ';' inside a quoted string for now. Can't see a use
- * for it
+ * A filter def can look like:
+ *      someprog -v -t " h i j";charset= xx; mimetype=yy
+ * A semi-colon list of attr=value pairs may come after the exec spec.
+ * This list is treated by replacing semi-colons with newlines and building
+ * a confsimple. This is done quite brutally and we don't support having
+ * a ';' inside a quoted string for now. Can't see a use for it.
  */
-MimeHandlerExec *mhExecFactory(RclConfig *cfg, const string& mtype, string& hs)
+MimeHandlerExec *mhExecFactory(RclConfig *cfg, const string& mtype, string& hs,
+                               bool multiple)
 {
-    list<string>semicolist;
-    stringToTokens(hs, semicolist, ";");
-    if (hs.size() < 1) {
-	LOGERR(("mhExecFactory: bad filter def: [%s]\n", hs.c_str()));
-	return 0;
+    ConfSimple attrs;
+    string cmdstr;
+    if (!cfg->valueSplitAttributes(hs, cmdstr, attrs)) {
+	LOGERR(("mhExecFactory: bad config line for [%s]: [%s]\n", 
+		mtype.c_str(), hs.c_str()));
+        return 0;
     }
-    string& cmd = *(semicolist.begin());
 
-    list<string> toks;
-    stringToStrings(cmd, toks);
-    if (toks.size() < 2) {
+    // Split command name and args, and build exec object
+    list<string> cmdtoks;
+    stringToStrings(cmdstr, cmdtoks);
+    if (cmdtoks.empty()) {
 	LOGERR(("mhExecFactory: bad config line for [%s]: [%s]\n", 
 		mtype.c_str(), hs.c_str()));
 	return 0;
     }
-
-    MimeHandlerExec *h = new MimeHandlerExec(mtype.c_str());
-
-    list<string>::iterator it;
-
-    // toks size is at least 2, this has been checked by caller.
-    it = toks.begin();
-    it++;
+    MimeHandlerExec *h = multiple ? 
+        new MimeHandlerExecMultiple(mtype.c_str()) :
+        new MimeHandlerExec(mtype.c_str());
+    list<string>::iterator it = cmdtoks.begin();
     h->params.push_back(cfg->findFilter(*it++));
-    h->params.insert(h->params.end(), it, toks.end());
+    h->params.insert(h->params.end(), it, cmdtoks.end());
 
-    // Handle additional parameters
-    it = semicolist.begin();
-    it++;
-    for (;it != semicolist.end(); it++) {
-	string &line = *it;
-	string::size_type eqpos = line.find("=");
-	if (eqpos == string::npos)
-	    continue;
-	// Compute name and value, trim white space
-	string nm, val;
-	nm = line.substr(0, eqpos);
-	trimstring(nm);
-	val = line.substr(eqpos+1, string::npos);
-	trimstring(val);
-	if (!nm.compare("charset")) {
-	    h->cfgCharset = stringtolower((const string&)val);
-	} else if (!nm.compare("mimetype")) {
-	    h->cfgMtype = stringtolower((const string&)val);
-	}
-    }
+    // Handle additional attributes. We substitute the semi-colons
+    // with newlines and use a ConfSimple
+    string value;
+    if (attrs.get("charset", value)) 
+        h->cfgCharset = stringtolower((const string&)value);
+    if (attrs.get("mimetype", value))
+        h->cfgMtype = stringtolower((const string&)value);
 
-#if 0
-    string sparams;
+#if 1
+    string scmd;
     for (it = h->params.begin(); it != h->params.end(); it++) {
-	sparams += string("[") + *it + "] ";
+	scmd += string("[") + *it + "] ";
     }
-    LOGDEB(("mhExecFactory:mt [%s] cfgmt [%s] cfgcs [%s] params: [%s]\n",
+    LOGDEB(("mhExecFactory:mt [%s] cfgmt [%s] cfgcs [%s] cmd: [%s]\n",
 	    mtype.c_str(), h->cfgMtype.c_str(), h->cfgCharset.c_str(),
-	    sparams.c_str()));
+	    scmd.c_str()));
 #endif
 
     return h;
@@ -195,33 +157,44 @@ Dijon::Filter *getMimeHandler(const string &mtype, RclConfig *cfg,
     hs = cfg->getMimeHandlerDef(mtype, filtertypes);
 
     if (!hs.empty()) {
-	// Break definition into type and name 
-	list<string> toks;
-	stringToStrings(hs, toks);
-	if (toks.empty()) {
-	    LOGERR(("getMimeHandler: bad mimeconf line for %s\n", 
-		    mtype.c_str()));
-	    return 0;
-	}
-
-	// Retrieve handler function according to type
-	list<string>::iterator it = toks.begin();
-	if (!stringlowercmp("internal", *it)) {
+	// Break definition into type and name/command string
+        string::size_type b1 = hs.find_first_of(" \t");
+        string handlertype = hs.substr(0, b1);
+	if (!stringlowercmp("internal", handlertype)) {
 	    return mhFactory(mtype);
-	} else if (!stringlowercmp("dll", *it)) {
-	} else if (!stringlowercmp("exec", *it)) {
-	    if (toks.size() < 2) {
+	} else if (!stringlowercmp("dll", handlertype)) {
+	} else {
+            string cmdstr = hs.substr(b1);
+            trimstring(cmdstr);
+            if (cmdstr.empty()) {
 		LOGERR(("getMimeHandler: bad line for %s: %s\n", 
 			mtype.c_str(), hs.c_str()));
 		return 0;
 	    }
-	    return mhExecFactory(cfg, mtype, hs);
+            if (!stringlowercmp("exec", handlertype)) {
+                return mhExecFactory(cfg, mtype, cmdstr, false);
+            } else if (!stringlowercmp("execm", handlertype)) {
+                return mhExecFactory(cfg, mtype, cmdstr, true);
+            } else {
+		LOGERR(("getMimeHandler: bad line for %s: %s\n", 
+			mtype.c_str(), hs.c_str()));
+		return 0;
+            }
 	}
     }
 
-    // We are supposed to get here if there was no specific error, but
-    // there is no identified mime type, or no handler
-    // associated. These files are either ignored or their name is
+    // We get here if there was no specific error, but there is no
+    // identified mime type, or no handler associated.
+
+#ifdef INDEX_UNKNOWN_TEXT_AS_PLAIN
+    // If the type is an unknown text/xxx, index as text/plain and
+    // hope for the best (this wouldn't work too well with text/rtf...)
+    if (mtype.find("text/") == 0) {
+        return mhFactory("text/plain");
+    }
+#endif
+
+    // Finally, unhandled files are either ignored or their name is
     // indexed, depending on configuration
     bool indexunknown = false;
     cfg->getConfParam("indexallfilenames", &indexunknown);
