@@ -23,10 +23,6 @@
 #include <list>
 using std::string;
 using std::list;
-#ifdef RCL_USE_XATTR
-#include <map>
-using std::map;
-#endif // RCL_USE_XATTR
 
 #include <Filter.h>
 
@@ -40,6 +36,9 @@ public:
     virtual ~RecollFilter() {}
     virtual bool set_property(Properties p, const string &v) {
 	switch (p) {
+	case DJF_UDI: 
+	    m_udi = v;
+	    break;
 	case DEFAULT_CHARSET: 
 	    m_defcharset = v;
 	    break;
@@ -56,9 +55,10 @@ public:
     // We don't use this for now
     virtual bool set_document_uri(const std::string &) {return false;}
 
-    /// This does the extended attributes thing if enabled and should
-    /// be called from subclasses.
-    virtual bool set_document_file(const string &file_path);
+    /// This does nothing right now but should be called from the
+    /// subclass method in case we need some common processing one day
+    /// (was used for xattrs at some point).
+    virtual bool set_document_file(const string & /*file_path*/) {return true;}
 
     // Default implementations
     virtual bool set_document_string(const std::string &) {return false;}
@@ -90,22 +90,14 @@ public:
 	m_forPreview = m_havedoc = false;
 	m_defcharset.clear();
 	m_reason.clear();
-#ifdef RCL_USE_XATTR
-	m_fieldsFromAttrs.clear();
-#endif // RCL_USE_XATTR
     }
-#ifdef RCL_USE_XATTR
-    const map<string, string>& getFieldsFromAttrs() {return m_fieldsFromAttrs;}
-#endif // RCL_USE_XATTR
 
 protected:
     bool   m_forPreview;
     string m_defcharset;
     string m_reason;
     bool   m_havedoc;
-#ifdef RCL_USE_XATTR
-    map<string, string> m_fieldsFromAttrs;
-#endif // RCL_USE_XATTR
+    string m_udi; // May be set by creator as a hint
 };
 
 /**
