@@ -2,6 +2,11 @@
 #set -x
 # A shell-script to make a recoll static binary distribution:
 
+fatal()
+{
+    echo $*;exit 1
+}
+
 TAR=tar
  
 targetdir=${targetdir-/tmp}
@@ -15,6 +20,14 @@ version=`cat VERSION`
 sys=`uname -s`
 sysrel=`uname -r`
 
+qtguiassign=`egrep '^QTGUI=' recollinstall`
+stripassign=`egrep '^STRIP=' recollinstall`
+test ! -z "$qtguiassign" || fatal "Can't find qt version"
+test ! -z "$stripassign" || fatal "Can't find strip string"
+eval $qtguiassign
+eval $stripassign
+echo "QTGUI: " $QTGUI "STRIP: " $STRIP
+
 topdirsimple=recoll-${version}-${sys}-${sysrel}
 topdir=$targetdir/$topdirsimple
 
@@ -27,28 +40,40 @@ else
     read rep 
     if test $rep = 'y';then
     	rm -rf $topdir/*
+    else
+	exit 1
     fi
 fi
 
+rm -f index/recollindex ${QTGUI}/recoll
 
-rm -f index/recollindex qtgui/recoll
 make static || exit 1
-strip index/recollindex qtgui/recoll
+
+${STRIP} index/recollindex ${QTGUI}/recoll
 
 files="COPYING README INSTALL VERSION Makefile recollinstall
 filters desktop sampleconf doc/user doc/man
-index/recollindex index/rclmon.sh qtgui/recoll qtgui/i18n/*.qm 
+index/recollindex index/rclmon.sh ${QTGUI}/recoll qtgui/i18n/*.qm 
 qtgui/mtpics/*.png 
 desktop/recoll.png desktop/recoll-searchgui.desktop"
 
 $TAR chf - $files  | (cd $topdir; $TAR xf -)
 
 # Remove any install dependancy
-chmod +w $topdir/Makefile
+chmod +w $topdir/Makefile || exit 1
 sed -e '/^install:/c\
 install: ' < $topdir/Makefile > $topdir/toto && \
 	 mv $topdir/toto $topdir/Makefile
 
+# Clean up .svn directories from target. This would be easier with a
+# --exclude tar option, but we want this to work with non-gnu tars
+cd $topdir || exit 1
+svndirs=`find . -name .svn -print`
+echo "In: `pwd`. Removing $svndirs ok ?"
+read rep 
+test "$rep" = 'y' -o "$rep" = 'Y' && rm -rf $svndirs
+
+cd $targetdir
 
 (cd $targetdir ; \
     $TAR chf - $topdirsimple | \
