@@ -44,7 +44,7 @@ module1 = Extension('recoll',
                                os.path.join(top, 'common/unacpp.cpp'),
                                os.path.join(top, 'query/wasastringtoquery.cpp'),
                                os.path.join(top, 'query/wasatorcl.cpp'),
-                               os.path.join(top, 'rcldb/pathhash.cpp'),
+                               os.path.join(top, 'utils/fileudi.cpp'),
                                os.path.join(top, 'rcldb/rcldb.cpp'),
                                os.path.join(top, 'rcldb/rcldoc.cpp'),
                                os.path.join(top, 'rcldb/rclquery.cpp'),
