@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "@(#$Id: advsearch_w.cpp,v 1.21 2008/09/25 06:00:24 dockes Exp $ (C) 2005 J.F.Dockes";
+static char rcsid[] = "@(#$Id: advsearch_w.cpp,v 1.21 2008-09-25 06:00:24 dockes Exp $ (C) 2005 J.F.Dockes";
 #endif
 /*
  *   This program is free software; you can redistribute it and/or modify
@@ -25,13 +25,16 @@ static char rcsid[] = "@(#$Id: advsearch_w.cpp,v 1.21 2008/09/25 06:00:24 dockes
 #include <qlineedit.h>
 #include <qframe.h>
 #include <qcheckbox.h>
+#include <qevent.h>
 
 #if (QT_VERSION < 0x040000)
 #include <qcombobox.h>
 #include <qlistbox.h>
+#define Q34EVOVERRIDE QEvent::AccelOverride
 #else
 #include <q3combobox.h>
 #include <q3listbox.h>
+#define Q34EVOVERRIDE QEvent::ShortcutOverride
 #endif
 
 #include <qlayout.h>
@@ -57,6 +60,7 @@ using std::unique;
 #include "debuglog.h"
 #include "searchdata.h"
 #include "guiutils.h"
+#include "rclhelp.h"
 
 extern RclConfig *rclconfig;
 
@@ -68,6 +72,11 @@ static map<QString,QString> cat_rtranslations;
 
 void AdvSearch::init()
 {
+    (void)new HelpClient(this);
+    HelpClient::installMap(this->name(), "RCL.SEARCH.COMPLEX");
+
+    this->installEventFilter(this);
+
     // signals and slots connections
     connect(delFiltypPB, SIGNAL(clicked()), this, SLOT(delFiltypPB_clicked()));
     connect(searchPB, SIGNAL(clicked()), this, SLOT(runSearch()));
@@ -108,6 +117,7 @@ void AdvSearch::init()
 	    }
 	}
     }
+    (*m_clauseWins.begin())->wordsLE->setFocus();
 
     // Initialize lists of accepted and ignored mime types from config
     // and settings
@@ -146,6 +156,19 @@ void AdvSearch::init()
     cat_rtranslations[tr("other")] = QString::fromUtf8("other"); 
 }
 
+bool AdvSearch::eventFilter(QObject *, QEvent *event)
+{
+    //    LOGDEB(("AdvSearch::eventFilter. Type %d\n", (int)event->type()));
+    if (event->type() == QEvent::KeyPress || event->type() == Q34EVOVERRIDE) {
+	QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+	if (ke->key() == Qt::Key_Q && (ke->state() & Qt::ControlButton)) {
+	    recollNeedsExit = 1;
+	    return true;
+	}
+    }
+    return false;
+}
+
 void AdvSearch::saveCnf()
 {
     // Save my state
@@ -181,9 +204,6 @@ void AdvSearch::addClause()
     addClause(0);
 }
 
-#define HORADJ 50
-#define VERTADJ 30
-
 void AdvSearch::addClause(int tp)
 {
     SearchClauseW *w = new SearchClauseW(clauseFRM);
@@ -196,9 +216,6 @@ void AdvSearch::addClause(int tp)
     } else {
 	delClausePB->setEnabled(false);
     }
-    // Have to adjust the size else we lose the bottom buttons! Why?
-    QSize sz = layout()->sizeHint();
-    resize(QSize(sz.width()+HORADJ, sz.height()+VERTADJ));
 }
 
 void AdvSearch::delClause()
@@ -212,20 +229,8 @@ void AdvSearch::delClause()
     } else {
 	delClausePB->setEnabled(false);
     }
-    QSize sz = layout()->sizeHint();
-    resize(QSize(sz.width()+HORADJ, sz.height()+VERTADJ));
 }
 
-#if (QT_VERSION < 0x040000)
-void AdvSearch::polish()
-{
-    AdvSearchBase::polish();
-    QSize sz = sizeHint();
-    resize(QSize(sz.width()+HORADJ+10, sz.height()+VERTADJ-20));
-}
-#endif
-
-
 // Move selected file types from the searched to the ignored box
 void AdvSearch::delFiltypPB_clicked()
 {
