@@ -1,6 +1,6 @@
 #ifndef _ADVSEARCH_W_H_INCLUDED_
 #define _ADVSEARCH_W_H_INCLUDED_
-/* @(#$Id: advsearch_w.h,v 1.13 2007/08/31 09:04:23 dockes Exp $  (C) 2005 J.F.Dockes */
+/* @(#$Id: advsearch_w.h,v 1.13 2007-08-31 09:04:23 dockes Exp $  (C) 2005 J.F.Dockes */
 /*
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@ public:
 	init();
     }
     ~AdvSearch(){}
+    virtual bool eventFilter(QObject *target, QEvent *event);
 
 public slots:
     virtual void delFiltypPB_clicked();
@@ -74,9 +75,6 @@ public slots:
     virtual void addClause();
     virtual void addClause(int);
     virtual bool close();
-#if (QT_VERSION < 0x040000)
-    virtual void polish();
-#endif
 
 signals:
     void startSearch(RefCntr<Rcl::SearchData>);
