@@ -45,6 +45,7 @@ class QLISTBOX;
 class QSpinBox;
 class QComboBox;
 class QCheckBox;
+class QPushButton;
 
 namespace confgui {
 
@@ -76,29 +77,32 @@ namespace confgui {
 	Q_OBJECT
     public:
 	ConfParamW(QWidget *parent, ConfLink cflink)
-	    : QWidget(parent), m_cflink(cflink)
+	    : QWidget(parent), m_cflink(cflink), m_fsencoding(false)
 	{
 	}
 	virtual void loadValue() = 0;
-
+        virtual void setFsEncoding(bool onoff) {m_fsencoding = onoff;}
     protected:
 	ConfLink     m_cflink;
 	QHBOXLAYOUT *m_hl;
-
+        // File names are encoded as local8bit in the config files. Other
+        // are encoded as utf-8
+        bool         m_fsencoding;
 	virtual bool createCommon(const QString& lbltxt,
 				  const QString& tltptxt);
-
+    public slots:
+        virtual void setEnabled(bool) = 0;
     protected slots:
         void setValue(const QString& newvalue);
         void setValue(int newvalue);
         void setValue(bool newvalue);
     };
 
-
     // Widgets for setting the different types of configuration parameters:
 
     // Int
     class ConfParamIntW : public ConfParamW {
+	Q_OBJECT
     public:
 	ConfParamIntW(QWidget *parent, ConfLink cflink, 
 		      const QString& lbltxt,
@@ -106,40 +110,51 @@ namespace confgui {
 		      int minvalue = INT_MIN, 
 		      int maxvalue = INT_MAX);
 	virtual void loadValue();
+    public slots:
+        virtual void setEnabled(bool i) {if(m_sb) ((QWidget*)m_sb)->setEnabled(i);}
     protected:
 	QSpinBox *m_sb;
     };
 
     // Arbitrary string
     class ConfParamStrW : public ConfParamW {
+	Q_OBJECT
     public:
 	ConfParamStrW(QWidget *parent, ConfLink cflink, 
 		      const QString& lbltxt,
 		      const QString& tltptxt);
 	virtual void loadValue();
+    public slots:
+        virtual void setEnabled(bool i) {if(m_le) ((QWidget*)m_le)->setEnabled(i);}
     protected:
 	QLineEdit *m_le;
     };
 
     // Constrained string: choose from list
     class ConfParamCStrW : public ConfParamW {
+	Q_OBJECT
     public:
 	ConfParamCStrW(QWidget *parent, ConfLink cflink, 
 		      const QString& lbltxt,
 		      const QString& tltptxt, const QStringList& sl);
 	virtual void loadValue();
+    public slots:
+        virtual void setEnabled(bool i) {if(m_cmb) ((QWidget*)m_cmb)->setEnabled(i);}
     protected:
 	QComboBox *m_cmb;
     };
 
     // Boolean
     class ConfParamBoolW : public ConfParamW {
+	Q_OBJECT
     public:
 	ConfParamBoolW(QWidget *parent, ConfLink cflink, 
 		      const QString& lbltxt,
 		      const QString& tltptxt);
 	virtual void loadValue();
-    protected:
+    public slots:
+        virtual void setEnabled(bool i) {if(m_cb) ((QWidget*)m_cb)->setEnabled(i);}
+    public:
 	QCheckBox *m_cb;
     };
 
@@ -153,8 +168,15 @@ namespace confgui {
 	virtual void loadValue();
     protected slots:
 	void showBrowserDialog();
+    public slots:
+        virtual void setEnabled(bool i) 
+        {
+            if(m_le) ((QWidget*)m_le)->setEnabled(i);
+            if(m_pb) ((QWidget*)m_pb)->setEnabled(i);
+        }
     protected:
 	QLineEdit *m_le;
+        QPushButton *m_pb;
 	bool       m_isdir;
     };
 
@@ -168,6 +190,8 @@ namespace confgui {
 	virtual void loadValue();
 	QLISTBOX *getListBox() {return m_lb;}
 	
+    public slots:
+        virtual void setEnabled(bool i) {if(m_lb) ((QWidget*)m_lb)->setEnabled(i);}
     protected slots:
 	virtual void showInputDialog();
 	void deleteSelected();
@@ -176,7 +200,6 @@ namespace confgui {
     protected:
 	QLISTBOX *m_lb;
 	void listToConf();
-	
     };
 
     // Dir name list
@@ -188,6 +211,7 @@ namespace confgui {
 		      const QString& tltptxt)
 	    : ConfParamSLW(parent, cflink, lbltxt, tltptxt)
 	    {
+                m_fsencoding = true;
 	    }
     protected slots:
 	virtual void showInputDialog();
