@@ -271,7 +271,7 @@ Drücken Sie Abbrechen, um die Konfigurationsdatei vor dem Start der Indizierung
     </message>
     <message>
         <source>File does not exist</source>
-        <translation>Datei existiert nicht</translation>
+        <translation type="obsolete">Datei existiert nicht</translation>
     </message>
     <message>
         <source>Missing helper program: </source>
@@ -335,6 +335,14 @@ Drücken Sie Abbrechen, um die Konfigurationsdatei vor dem Start der Indizierung
         <source>Show main text</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Print</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Print Current Preview</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>QObject</name>
@@ -390,6 +398,10 @@ Drücken Sie Abbrechen, um die Konfigurationsdatei vor dem Start der Indizierung
         <source>Index the names of files for which the contents cannot be identified or processed (no or unsupported mime type). Default true</source>
         <translation>Indiziere alle Namen der Dateien, deren Inhalt nicht bestimmt oder bearbeitet werden kann (keiner oder nicht unterstützter mime Typ). Standard: ja</translation>
     </message>
+    <message>
+        <source>Beagle web history</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>QtGuiResListPager</name>
@@ -422,7 +434,7 @@ Drücken Sie Abbrechen, um die Konfigurationsdatei vor dem Start der Indizierung
     </message>
     <message>
         <source>No external viewer configured for mime type </source>
-        <translation>Kein externer Betrachter für Mime Type </translation>
+        <translation type="obsolete">Kein externer Betrachter für Mime Type </translation>
     </message>
     <message>
         <source>Executing: [</source>
@@ -434,7 +446,7 @@ Drücken Sie Abbrechen, um die Konfigurationsdatei vor dem Start der Indizierung
     </message>
     <message>
         <source>Cannot access document file: </source>
-        <translation>Fehler beim Zugriff auf das Dokument:</translation>
+        <translation type="obsolete">Fehler beim Zugriff auf das Dokument:</translation>
     </message>
     <message>
         <source>Preview</source>
@@ -539,7 +551,7 @@ Please check the mimeconf file</source>
     <message>
         <source>The viewer specified in mimeconf for %1: %2 is not found.
 Do you want to start the  preferences dialog ?</source>
-        <translation>Der in der mimeconf konfigurierte Betrachter für %1: %2 wurde nicht gefunden.
+        <translation type="obsolete">Der in der mimeconf konfigurierte Betrachter für %1: %2 wurde nicht gefunden.
 Soll der Voreinstellungsdialog geöffnet werden?</translation>
     </message>
     <message>
@@ -628,6 +640,19 @@ Soll der Voreinstellungsdialog geöffnet werden?</translation>
         <source>Choose a file name to save under</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Document category filter</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No external viewer configured for mime type [</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The viewer specified in mimeview for %1: %2 is not found.
+Do you want to start the  preferences dialog ?</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>RclMainBase</name>
@@ -947,7 +972,7 @@ Soll der Voreinstellungsdialog geöffnet werden?</translation>
     </message>
     <message>
         <source>P&amp;arent document/folder</source>
-        <translation>&amp;übergeordnetes Dokument/Ordner</translation>
+        <translation type="obsolete">&amp;übergeordnetes Dokument/Ordner</translation>
     </message>
     <message>
         <source>Documents &lt;b&gt;%1-%2&lt;/b&gt; out of at least &lt;b&gt;%3&lt;/b&gt; for </source>
@@ -969,10 +994,6 @@ Soll der Voreinstellungsdialog geöffnet werden?</translation>
         <source>Document history</source>
         <translation type="unfinished">Dokumenthistorie</translation>
     </message>
-    <message>
-        <source>Save to File</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Documents &lt;b&gt;%d-%d&lt;/b&gt; out of at least &lt;b&gt;%d&lt;/b&gt; for </source>
         <translation type="unfinished"></translation>
@@ -989,6 +1010,22 @@ Soll der Voreinstellungsdialog geöffnet werden?</translation>
         <source>Open</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>&lt;p&gt;&lt;i&gt;Alternate spellings (accents suppressed): &lt;/i&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Write to File</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Preview P&amp;arent document/folder</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Open Parent document/folder</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ResListBase</name>
@@ -1754,6 +1791,10 @@ Dadurch sollten Ergebnisse, die exakte Übereinstimmungen der Suchworte enthalte
         <source>Choose editor applications</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Display category filter as toolbar instead of button panel (needs restart).</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ViewAction</name>
@@ -1797,7 +1838,7 @@ Dadurch sollten Ergebnisse, die exakte Übereinstimmungen der Suchworte enthalte
         <translation>Anzeigeprogramme</translation>
     </message>
     <message>
-        <source>Select one or several mime types then click &quot;Change Action&quot;</source>
+        <source>Select one or several mime types then click &quot;Change Action&quot;&lt;br&gt;You can also close this dialog and check &quot;Use desktop preferences&quot;&lt;br&gt;in the main panel to ignore this list and use your desktop defaults.</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -1888,6 +1929,33 @@ Dadurch sollten Ergebnisse, die exakte Übereinstimmungen der Suchworte enthalte
         <translation type="obsolete">Impossible d&apos;éxécuter une requête purement négative. Essayez d&apos;entrer des termes courants dans un champ &apos;ou&apos;</translation>
     </message>
 </context>
+<context>
+    <name>confgui::ConfBeaglePanelW</name>
+    <message>
+        <source>Steal Beagle indexing queue</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Beagle MUST NOT be running. Enables processing the beagle queue to index Firefox web history.&lt;br&gt;(you should also install the Firefox Beagle plugin)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Web cache directory name</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The name for a directory where to store the cache for visited web pages.&lt;br&gt;A non-absolute path is taken relative to the configuration directory.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Max. size for the web cache (MB)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Entries will be recycled once the size is reached</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 <context>
     <name>confgui::ConfIndexW</name>
     <message>
@@ -1927,6 +1995,33 @@ Dadurch sollten Ergebnisse, die exakte Übereinstimmungen der Suchworte enthalte
         <source>This value sets a threshold beyond which compressedfiles will not be processed. Set to -1 for no limit, to 0 for no decompression ever.</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Max. text file size (MB)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>This value sets a threshold beyond which text files will not be processed. Set to -1 for no limit. 
+This is for excluding monster log files from the index.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Text file page size (KB)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>If this value is set (not equal to -1), text files will be split in chunks of this size for indexing.
+This will help searching very big text  files (ie: log files).</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Max. filter exec. time (S)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>External filters working longer than this will be aborted. This is for the rare case (ie: postscript) where a document could cause a filter to loopSet to -1 for no limit.
+</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>confgui::ConfTopPanelW</name>
