@@ -37,8 +37,8 @@ static char rcsid[] = "@(#$Id: main.cpp,v 1.73 2008-12-17 08:01:40 dockes Exp $
 #endif
 
 #include <qtextcodec.h> 
-#include <qthread.h>
 #include <qtimer.h>
+#include <qthread.h>
 #include <qmessagebox.h>
 #include <qcheckbox.h>
 #include <qcombobox.h>
@@ -51,7 +51,6 @@ static char rcsid[] = "@(#$Id: main.cpp,v 1.73 2008-12-17 08:01:40 dockes Exp $
 #include "rclinit.h"
 #include "debuglog.h"
 #ifdef WITH_KDE
-#include "rclversion.h"
 #endif
 #include "rclmain_w.h"
 #include "ssearch_w.h"
@@ -84,7 +83,7 @@ RclConfig* RclConfig::getMainConfig()
     return rclconfig;
 }
 
-RclHistory *g_dynconf;
+RclDynConf *g_dynconf;
 int recollNeedsExit;
 int startIndexingAfterConfig;
 RclMain *mainWindow;
@@ -124,8 +123,6 @@ static void recollCleanup()
 {
     LOGDEB(("recollCleanup: writing settings\n"));
     rwSettings(true);
-    LOGDEB2(("recollCleanup: stopping idx thread\n"));
-    stop_idxthread();
     LOGDEB2(("recollCleanup: closing database\n"));
     deleteZ(rcldb);
     deleteZ(rclconfig);
@@ -174,7 +171,7 @@ static const char usage [] =
 "     -f : the query will be interpreted as a filename search\n"
 "     -l : the query will be interpreted as a query language string (default)\n"
 "     -t : terminal display: no gui. Results go to stdout. MUST be given\n"
-"          explicitely as -t (not ie, -at), and -q <query> MUST\n"
+"          explicitly as -t (not ie, -at), and -q <query> MUST\n"
 "          be last on the command line if this is used.\n"
 ;
 static void
@@ -194,7 +191,8 @@ int main(int argc, char **argv)
     }
 
 #ifdef WITH_KDE
-    KAboutData about("recoll", I18N_NOOP("Recoll"), rclversion, description,
+    KAboutData about("recoll", I18N_NOOP("Recoll"), Rcl::version_string(), 
+                     description,
                      KAboutData::License_GPL, "(C) 2006 Jean-Francois Dockes", 0, 0, "jean-francois.dockes@wanadoo.fr");
     about.addAuthor( "Jean-Francois Dockes", 0, 
 		     "jean-francois.dockes@wanadoo.fr" );
@@ -240,7 +238,7 @@ int main(int argc, char **argv)
     if (op_flags & OPT_q)
 	while (argc--) {
 	    question += " ";
-	    question += *argv--;
+	    question += *argv++;
 	}
 
     // Translation file for Qt
@@ -278,7 +276,7 @@ int main(int argc, char **argv)
 #endif
 
     string historyfile = path_cat(rclconfig->getConfDir(), "history");
-    g_dynconf = new RclHistory(historyfile);
+    g_dynconf = new RclDynConf(historyfile);
     if (!g_dynconf || !g_dynconf->ok()) {
 	QString msg = app.translate("Main", "Configuration problem (dynconf");
 	QMessageBox::critical(0, "Recoll",  msg);
@@ -318,12 +316,8 @@ int main(int argc, char **argv)
 	mainWindow->resize(s);
     }
 
-    mainWindow->sSearch->searchTypCMB->setCurrentItem(prefs.ssearchTyp);
-    mainWindow->sSearch->searchTypeChanged(prefs.ssearchTyp);
     string dbdir = rclconfig->getDbDir();
     if (dbdir.empty()) {
-	// Note: this will have to be replaced by a call to a
-	// configuration buildin dialog for initial configuration
 	QMessageBox::critical(0, "Recoll",
 			      app.translate("Main", 
 					  "No db directory in configuration"));
@@ -332,55 +326,20 @@ int main(int argc, char **argv)
 
     rcldb = new Rcl::Db(rclconfig);
 
-    bool needindexconfig = false;
-    if (!maybeOpenDb(reason)) {
-
-	switch (QMessageBox::
-#if (QT_VERSION >= 0x030200)
-		question
-#else
-		information
-#endif
-		(0, "Recoll",
-		 app.translate("Main", "Could not open database in ") +
-		 QString::fromLocal8Bit(dbdir.c_str()) +
-		 app.translate("Main", 
-			       ".\n"
-			       "Click Cancel if you want to edit the configuration file before indexation starts, or Ok to let it proceed."),
-		 "Ok", "Cancel", 0,   0)) {
-
-	case 0: // Ok: indexing is going to start.
-	    start_indexing(true);
-	    break;
-
-	case 1: // Cancel
-	    needindexconfig = true;
-	    break;
-	}
-    }
-
     mainWindow->show();
-    if (needindexconfig) {
-	startIndexingAfterConfig = 1;
-	mainWindow->showIndexConfig();
-    } else {
-	if (prefs.startWithAdvSearchOpen)
-	    mainWindow->showAdvSearchDialog();
-	if (prefs.startWithSortToolOpen)
-	    mainWindow->showSortDialog();
-    }
+    QTimer::singleShot(0, mainWindow, SLOT(initDbOpen()));
 
     // Connect exit handlers etc.. Beware, apparently this must come
-    // after mainWindow->show() , else the QMessageBox above never
-    // returns.
+    // after mainWindow->show()?
     app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));
-    QTimer *timer = new QTimer(&app);
-    mainWindow->connect(timer, SIGNAL(timeout()), 
-			mainWindow, SLOT(periodic100()));
-    timer->start(100);
     app.connect(&app, SIGNAL(aboutToQuit()), mainWindow, SLOT(close()));
 
+    // Start the indexing thread. It will immediately go to sleep waiting for 
+    // something to do.
     start_idxthread(*rclconfig);
+
+    mainWindow->sSearch->searchTypCMB->setCurrentItem(prefs.ssearchTyp);
+    mainWindow->sSearch->searchTypeChanged(prefs.ssearchTyp);
     if (op_flags & OPT_q) {
 	SSearch::SSearchType stype;
 	if (op_flags & OPT_o) {
@@ -395,12 +354,7 @@ int main(int argc, char **argv)
 	mainWindow->sSearch->searchTypCMB->setCurrentItem(int(stype));
 	mainWindow->
 	    sSearch->setSearchString(QString::fromLocal8Bit(question.c_str()));
-	// The 200 ms are a hack to jump over the first db close by 
-	// periodic100()... 
-	QTimer::singleShot(200, mainWindow->sSearch, SLOT(startSimpleSearch()));
     }
-    //    fprintf(stderr, "Go\n");
-    // Let's go
     return app.exec();
 }
 
