@@ -33,18 +33,28 @@ using std::pair;
 #include <qvariant.h>
 #include <qpushbutton.h>
 #include <qtabwidget.h>
+#include <qprinter.h>
+#include <qprintdialog.h>
+
 #if (QT_VERSION < 0x040000)
 #include <qtextedit.h>
 #include <qpopupmenu.h>
 #include <qprogressdialog.h>
 #define THRFINISHED finished
+#include <qpaintdevicemetrics.h>
+#include <qsimplerichtext.h>
 #else
 #include <q3popupmenu.h>
 #include <q3textedit.h>
 #include <q3progressdialog.h>
 #include <q3stylesheet.h>
+#include <q3paintdevicemetrics.h>
+#define QPaintDeviceMetrics Q3PaintDeviceMetrics
+#include <q3simplerichtext.h>
+#define QSimpleRichText Q3SimpleRichText
 #define THRFINISHED isFinished
 #endif
+
 #include <qevent.h>
 #include <qlabel.h>
 #include <qlineedit.h>
@@ -65,6 +75,7 @@ using std::pair;
 #include "cancelcheck.h"
 #include "preview_w.h"
 #include "guiutils.h"
+#include "docseqhist.h"
 
 #if (QT_VERSION < 0x030300)
 #define wasCanceled wasCancelled
@@ -88,6 +99,10 @@ using std::pair;
 
 #include "rclhelp.h"
 
+#ifndef MIN
+#define MIN(A,B) ((A)<(B)?(A):(B))
+#endif
+
 // QTextEdit's scrollToAnchor() is supposed to make the anchor visible, but
 // actually, it only moves to the top of the paragraph containing the anchor.
 // As we only have one paragraph, this doesnt' help a lot (qt3 and qt4)
@@ -100,6 +115,7 @@ using std::pair;
 // even installed under qt4. We use a local copy, which is not nice.
 void PreviewTextEdit::moveToAnchor(const QString& name)
 {
+    LOGDEB0(("PreviewTextEdit::moveToAnchor\n"));
     if (name.isEmpty())
 	return;
     sync();
@@ -124,6 +140,111 @@ void PreviewTextEdit::moveToAnchor(const QString& name)
     }
 }
 
+
+#if (QT_VERSION >= 0x040000)
+
+// Had to reimplement Qtextdocument::find() by duplicating the qt3
+// version.  The version in qt4 qt3support was modified for some
+// unknown reason and exhibits quadratic behaviour, and is totally
+// unusable on big documents
+static bool QTextDocument_find(Q3TextDocument *doc, Q3TextCursor& cursor, 
+                               const QString &expr, bool cs, bool wo, 
+                               bool forward)
+{
+    Qt::CaseSensitivity caseSensitive = cs ? Qt::CaseSensitive : 
+        Qt::CaseInsensitive;
+
+    doc->removeSelection(Q3TextDocument::Standard);
+    Q3TextParagraph *p = 0;
+    if ( expr.isEmpty() )
+	return FALSE;
+    for (;;) {
+	if ( p != cursor.paragraph() ) {
+	    p = cursor.paragraph();
+	    QString s = cursor.paragraph()->string()->toString();
+	    int start = cursor.index();
+	    for ( ;; ) {
+		int res = forward ? s.indexOf(expr, start, caseSensitive ) : 
+                    s.lastIndexOf(expr, start, caseSensitive);
+		int end = res + expr.length();
+		if ( res == -1 || ( !forward && start <= res ) )
+		    break;
+		if (!wo || ((res == 0||s[res - 1].isSpace() || 
+                             s[res - 1].isPunct()) &&
+                            (end == (int)s.length() || s[end].isSpace() || 
+                             s[end].isPunct()))) {
+		    doc->removeSelection(Q3TextDocument::Standard);
+		    cursor.setIndex( forward ? end : res );
+		    doc->setSelectionStart(Q3TextDocument::Standard, cursor);
+		    cursor.setIndex( forward ? res : end );
+		    doc->setSelectionEnd(Q3TextDocument::Standard, cursor);
+		    if ( !forward )
+			cursor.setIndex( res );
+		    return TRUE;
+		}
+		start = res + (forward ? 1 : -1);
+	    }
+	}
+	if ( forward ) {
+	    if ( cursor.paragraph() == doc->lastParagraph() && 
+                 cursor.atParagEnd() )
+		 break;
+	    cursor.gotoNextLetter();
+	} else {
+	    if ( cursor.paragraph() == doc->firstParagraph() && 
+                 cursor.atParagStart() )
+		 break;
+	    cursor.gotoPreviousLetter();
+	}
+    }
+    return FALSE;
+
+}
+
+bool PreviewTextEdit::find(const QString &expr, bool cs, bool wo,
+                           bool forward, int *para, int *index)
+{
+    drawCursor(false);
+#ifndef QT_NO_CURSOR
+    viewport()->setCursor(isReadOnly() ? Qt::ArrowCursor : Qt::IBeamCursor);
+#endif
+    Q3TextCursor findcur = *textCursor();
+    if (para && index) {
+        if (document()->paragAt(*para))
+            findcur.gotoPosition(document()->paragAt(*para), *index);
+        else
+            findcur.gotoEnd();
+    } else if (document()->hasSelection(Q3TextDocument::Standard)){
+        // maks sure we do not find the same selection again
+        if (forward)
+            findcur.gotoNextLetter();
+        else
+            findcur.gotoPreviousLetter();
+    } else if (!forward && findcur.index() == 0 && findcur.paragraph() == findcur.topParagraph()) {
+        findcur.gotoEnd();
+    }
+    removeSelection(Q3TextDocument::Standard);
+    bool found = QTextDocument_find(document(), findcur, expr, cs, wo, forward);
+    if (found) {
+        if (para)
+            *para = findcur.paragraph()->paragId();
+        if (index)
+            *index = findcur.index();
+        *textCursor() = findcur;
+        repaintChanged();
+        ensureCursorVisible();
+    }
+    drawCursor(true);
+    if (found) {
+        emit cursorPositionChanged(textCursor());
+        emit cursorPositionChanged(textCursor()->paragraph()->paragId(), 
+                                   textCursor()->index());
+    }
+    return found;
+}
+
+#endif
+
 void Preview::init()
 {
     setName("Preview");
@@ -145,7 +266,6 @@ void Preview::init()
     pvEdit->setUndoRedoEnabled(FALSE);
     unnamedLayout->addWidget(pvEdit);
     pvTab->insertTab(unnamed, QString::fromLatin1(""));
-    m_tabData.push_back(TabData(pvTab->currentPage()));
 
     previewLayout->addWidget(pvTab);
 
@@ -235,10 +355,13 @@ void Preview::closeEvent(QCloseEvent *e)
 
 bool Preview::eventFilter(QObject *target, QEvent *event)
 {
+    LOGDEB2(("Preview::eventFilter()\n"));
     if (event->type() != QEvent::KeyPress) 
 	return false;
     
-    LOGDEB1(("Preview::eventFilter: keyEvent\n"));
+    LOGDEB2(("Preview::eventFilter: keyEvent\n"));
+
+    PreviewTextEdit *edit = currentEditor();
     QKeyEvent *keyEvent = (QKeyEvent *)event;
     if (keyEvent->key() == Qt::Key_Q && 
 	(keyEvent->state() & Qt::ControlButton)) {
@@ -250,22 +373,25 @@ bool Preview::eventFilter(QObject *target, QEvent *event)
     } else if (keyEvent->key() == Qt::Key_Down &&
 	       (keyEvent->state() & Qt::ShiftButton)) {
 	// LOGDEB(("Preview::eventFilter: got Shift-Up\n"));
-	TabData *d = tabDataForCurrent();
-	if (d) 
-	    emit(showNext(this, m_searchId, d->docnum));
+	if (edit) 
+	    emit(showNext(this, m_searchId, edit->m_data.docnum));
 	return true;
     } else if (keyEvent->key() == Qt::Key_Up &&
 	       (keyEvent->state() & Qt::ShiftButton)) {
 	// LOGDEB(("Preview::eventFilter: got Shift-Down\n"));
-	TabData *d = tabDataForCurrent();
-	if (d) 
-	    emit(showPrev(this, m_searchId, d->docnum));
+	if (edit) 
+	    emit(showPrev(this, m_searchId, edit->m_data.docnum));
 	return true;
     } else if (keyEvent->key() == Qt::Key_W &&
 	       (keyEvent->state() & Qt::ControlButton)) {
 	// LOGDEB(("Preview::eventFilter: got ^W\n"));
 	closeCurrentTab();
 	return true;
+    } else if (keyEvent->key() == Qt::Key_P &&
+	       (keyEvent->state() & Qt::ControlButton)) {
+	// LOGDEB(("Preview::eventFilter: got ^P\n"));
+	emit(printCurrentPreviewRequest());
+	return true;
     } else if (m_dynSearchActive) {
 	if (keyEvent->key() == Qt::Key_F3) {
 	    doSearch(searchTextLine->text(), true, false);
@@ -274,21 +400,16 @@ bool Preview::eventFilter(QObject *target, QEvent *event)
 	if (target != searchTextLine)
 	    return QApplication::sendEvent(searchTextLine, event);
     } else {
-	QWidget *tw = pvTab->currentPage();
-	PreviewTextEdit *e = 0;
-	if (tw)
-	    e = (PreviewTextEdit *)tw->child("pvEdit");
-	LOGDEB1(("Widget: %p, edit %p, target %p\n", tw, e, target));
-	if (e && target == e) {
+	if (edit && target == edit) {
 	    if (keyEvent->key() == Qt::Key_Slash) {
 		searchTextLine->setFocus();
 		m_dynSearchActive = true;
 		return true;
 	    } else if (keyEvent->key() == Qt::Key_Space) {
-		e->scrollBy(0, e->visibleHeight());
+		edit->scrollBy(0, edit->visibleHeight());
 		return true;
 	    } else if (keyEvent->key() == Qt::Key_BackSpace) {
-		e->scrollBy(0, -e->visibleHeight());
+		edit->scrollBy(0, -edit->visibleHeight());
 		return true;
 	    }
 	}
@@ -299,7 +420,7 @@ bool Preview::eventFilter(QObject *target, QEvent *event)
 
 void Preview::searchTextLine_textChanged(const QString & text)
 {
-    LOGDEB1(("search line text changed. text: '%s'\n", text.ascii()));
+    LOGDEB2(("search line text changed. text: '%s'\n", text.ascii()));
     if (text.isEmpty()) {
 	m_dynSearchActive = false;
 	//	nextButton->setEnabled(false);
@@ -319,12 +440,13 @@ void Preview::searchTextLine_textChanged(const QString & text)
 #define QStyleSheetItem Q3StyleSheetItem
 #endif
 
-PreviewTextEdit *Preview::getCurrentEditor()
+PreviewTextEdit *Preview::currentEditor()
 {
+    LOGDEB2(("Preview::currentEditor()\n"));
     QWidget *tw = pvTab->currentPage();
     PreviewTextEdit *edit = 0;
     if (tw) {
-	edit = (PreviewTextEdit*)tw->child("pvEdit");
+	edit = dynamic_cast<PreviewTextEdit*>(tw->child("pvEdit"));
     }
     return edit;
 }
@@ -337,12 +459,12 @@ void Preview::doSearch(const QString &_text, bool next, bool reverse,
 		       bool wordOnly)
 {
     LOGDEB(("Preview::doSearch: text [%s] txtlen %d next %d rev %d word %d\n", 
-	    (const char *)_text.utf8(), _text.length(), int(next), 
-	    int(reverse), int(wordOnly)));
+             (const char *)_text.utf8(), _text.length(), int(next), 
+             int(reverse), int(wordOnly)));
     QString text = _text;
 
     bool matchCase = matchCheck->isChecked();
-    PreviewTextEdit *edit = getCurrentEditor();
+    PreviewTextEdit *edit = currentEditor();
     if (edit == 0) {
 	// ??
 	return;
@@ -383,10 +505,11 @@ void Preview::doSearch(const QString &_text, bool next, bool reverse,
 	LOGDEB(("Preview::doSearch: setting cursor to %d %d\n", ps, is));
 	edit->setCursorPosition(ps, is);
     }
-
+    Chrono chron;
     LOGDEB(("Preview::doSearch: first find call\n"));
     bool found = edit->find(text, matchCase, wordOnly, !reverse, 0, 0);
-    LOGDEB(("Preview::doSearch: first find call return\n"));
+    LOGDEB(("Preview::doSearch: first find call return: %.2f S\n", 
+            chron.secs()));
     // If not found, try to wrap around. 
     if (!found && next) { 
 	LOGDEB(("Preview::doSearch: wrapping around\n"));
@@ -398,8 +521,10 @@ void Preview::doSearch(const QString &_text, bool next, bool reverse,
 	    mspara = msindex = 0;
 	}
 	LOGDEB(("Preview::doSearch: 2nd find call\n"));
+        chron.restart();
 	found = edit->find(text,matchCase, false, !reverse, &mspara, &msindex);
-	LOGDEB(("Preview::doSearch: 2nd find call return\n"));
+	LOGDEB(("Preview::doSearch: 2nd find call return %.2f S\n",
+                chron.secs()));
     }
 
     if (found) {
@@ -414,18 +539,22 @@ void Preview::doSearch(const QString &_text, bool next, bool reverse,
 
 void Preview::nextPressed()
 {
+    LOGDEB2(("PreviewTextEdit::nextPressed\n"));
     doSearch(searchTextLine->text(), true, false);
 }
 
 void Preview::prevPressed()
 {
+    LOGDEB2(("PreviewTextEdit::prevPressed\n"));
     doSearch(searchTextLine->text(), true, true);
 }
 
 // Called when user clicks on tab
 void Preview::currentChanged(QWidget * tw)
 {
-    QWidget *edit = (QWidget *)tw->child("pvEdit");
+    LOGDEB2(("PreviewTextEdit::currentChanged\n"));
+    PreviewTextEdit *edit = 
+        dynamic_cast<PreviewTextEdit*>(tw->child("pvEdit"));
     m_currentW = tw;
     LOGDEB1(("Preview::currentChanged(). Editor: %p\n", edit));
     
@@ -439,14 +568,16 @@ void Preview::currentChanged(QWidget * tw)
     disconnect(edit, SIGNAL(doubleClicked(int, int)), this, 0);
     connect(edit, SIGNAL(doubleClicked(int, int)), 
 	    this, SLOT(textDoubleClicked(int, int)));
+    // Disconnect the print signal and reconnect it to the current editor
+    LOGDEB(("Disconnecting reconnecting print signal\n"));
+    disconnect(this, SIGNAL(printCurrentPreviewRequest()), 0, 0);
+    connect(this, SIGNAL(printCurrentPreviewRequest()), edit, SLOT(print()));
 #if (QT_VERSION >= 0x040000)
     connect(edit, SIGNAL(selectionChanged()), this, SLOT(selecChanged()));
 #endif
     tw->installEventFilter(this);
     edit->installEventFilter(this);
-    TabData *d = tabDataForCurrent();
-    if (d) 
-	emit(previewExposed(this, m_searchId, d->docnum));
+    emit(previewExposed(this, m_searchId, edit->m_data.docnum));
 }
 
 #if (QT_VERSION >= 0x040000)
@@ -458,7 +589,7 @@ void Preview::currentChanged(QWidget * tw)
 // qt version.
 void Preview::selecChanged()
 {
-    LOGDEB1(("Selection changed\n"));
+    LOGDEB1(("Preview::selecChanged\n"));
     if (!m_currentW)
 	return;
     PreviewTextEdit *edit = (PreviewTextEdit*)m_currentW->child("pvEdit");
@@ -498,7 +629,7 @@ void Preview::textDoubleClicked(int, int)
 
 void Preview::closeCurrentTab()
 {
-    LOGDEB(("Preview::closeCurrentTab: m_loading %d\n", m_loading));
+    LOGDEB1(("Preview::closeCurrentTab: m_loading %d\n", m_loading));
     if (m_loading) {
 	CancelCheck::instance().setCancel();
 	return;
@@ -508,14 +639,6 @@ void Preview::closeCurrentTab()
 	if (!tw) 
 	    return;
 	pvTab->removePage(tw);
-	// Have to remove from tab data list
-	for (list<TabData>::iterator it = m_tabData.begin(); 
-	     it != m_tabData.end(); it++) {
-	    if (it->w == tw) {
-		m_tabData.erase(it);
-		return;
-	    }
-	}
     } else {
 	close();
     }
@@ -523,6 +646,7 @@ void Preview::closeCurrentTab()
 
 PreviewTextEdit *Preview::addEditorTab()
 {
+    LOGDEB1(("PreviewTextEdit::addEditorTab()\n"));
     QWidget *anon = new QWidget((QWidget *)pvTab);
     QVBoxLayout *anonLayout = new QVBoxLayout(anon, 1, 1, "anonLayout"); 
     PreviewTextEdit *editor = new PreviewTextEdit(anon, "pvEdit", this);
@@ -531,18 +655,20 @@ PreviewTextEdit *Preview::addEditorTab()
     anonLayout->addWidget(editor);
     pvTab->addTab(anon, "Tab");
     pvTab->showPage(anon);
-    m_tabData.push_back(TabData(anon));
     return editor;
 }
 
-void Preview::setCurTabProps(const string &fn, const Rcl::Doc &doc,
-			     int docnum)
+void Preview::setCurTabProps(const Rcl::Doc &doc, int docnum)
 {
+    LOGDEB1(("PreviewTextEdit::setCurTabProps\n"));
     QString title;
     map<string,string>::const_iterator meta_it;
-    if ((meta_it = doc.meta.find("title")) != doc.meta.end()) {
+    if ((meta_it = doc.meta.find(Rcl::Doc::keytt)) != doc.meta.end()
+        && !meta_it->second.empty()) {
 	    title = QString::fromUtf8(meta_it->second.c_str(), 
 				      meta_it->second.length());
+    } else {
+        title = QString::fromLocal8Bit(path_getsimple(doc.url).c_str());
     }
     if (title.length() > 20) {
 	title = title.left(10) + "..." + title.right(10);
@@ -567,47 +693,42 @@ void Preview::setCurTabProps(const string &fn, const Rcl::Doc &doc,
 	tiptxt += meta_it->second + "\n";
     pvTab->setTabToolTip(w,QString::fromUtf8(tiptxt.c_str(), tiptxt.length()));
 
-    TabData *d = tabDataForCurrent();
-    if (d) {
-	d->fn = fn;
-	d->ipath = doc.ipath;
-	d->docnum = docnum;
+    PreviewTextEdit *e = currentEditor();
+    if (e) {
+	e->m_data.url = doc.url;
+	e->m_data.ipath = doc.ipath;
+	e->m_data.docnum = docnum;
     }
 }
 
-TabData *Preview::tabDataForCurrent()
+bool Preview::makeDocCurrent(const Rcl::Doc& doc, int docnum, bool sametab)
 {
-    QWidget *w = pvTab->currentPage();
-    if (w == 0)
-	return 0;
-    for (list<TabData>::iterator it = m_tabData.begin(); 
-	 it != m_tabData.end(); it++) {
-	if (it->w == w) {
-	    return &(*it);
-	}
-    }
-    return 0;
-}
+    LOGDEB(("Preview::makeDocCurrent: %s\n", doc.url.c_str()));
 
-bool Preview::makeDocCurrent(const string &fn, size_t sz, 
-			     const Rcl::Doc& doc, int docnum, bool sametab)
-{
-    LOGDEB(("Preview::makeDocCurrent: %s\n", fn.c_str()));
-    for (list<TabData>::iterator it = m_tabData.begin(); 
-	 it != m_tabData.end(); it++) {
-	LOGDEB2(("Preview::makeFileCurrent: compare to w %p, file %s\n", 
-		 it->w, it->fn.c_str()));
-	if (!it->fn.compare(fn) && !it->ipath.compare(doc.ipath)) {
-	    pvTab->showPage(it->w);
-	    return true;
-	}
+    /* Check if we already have this page */
+    for (int i = 0; i < pvTab->count(); i++) {
+#if (QT_VERSION < 0x040000)
+        QWidget *tw = pvTab->page(i);
+#else
+        QWidget *tw = pvTab->widget(i);
+#endif
+        if (tw) {
+            PreviewTextEdit *edit = 
+                dynamic_cast<PreviewTextEdit*>(tw->child("pvEdit"));
+            if (edit && !edit->m_data.url.compare(doc.url) && 
+                !edit->m_data.ipath.compare(doc.ipath)) {
+                pvTab->showPage(tw);
+                return true;
+            }
+        }
     }
+
     // if just created the first tab was created during init
     if (!sametab && !m_justCreated && !addEditorTab()) {
 	return false;
     }
     m_justCreated = false;
-    if (!loadFileInCurrentTab(fn, sz, doc, docnum)) {
+    if (!loadDocInCurrentTab(doc, docnum)) {
 	closeCurrentTab();
 	return false;
     }
@@ -638,16 +759,15 @@ bool Preview::makeDocCurrent(const string &fn, size_t sz,
 /* A thread to to the file reading / format conversion */
 class LoadThread : public QThread {
     int *statusp;
-    Rcl::Doc *out;
+    Rcl::Doc& out;
+    const Rcl::Doc& idoc;
     string filename;
-    string ipath;
-    string *mtype;
     string tmpdir;
     int loglevel;
  public: 
     string missing;
-    LoadThread(int *stp, Rcl::Doc *odoc, string fn, string ip, string *mt) 
-	: statusp(stp), out(odoc), filename(fn), ipath(ip), mtype(mt) 
+    LoadThread(int *stp, Rcl::Doc& odoc, const Rcl::Doc& idc) 
+	: statusp(stp), out(odoc), idoc(idc)
 	{
 	    loglevel = DebugLog::getdbl()->getlevel();
 	}
@@ -667,36 +787,35 @@ class LoadThread : public QThread {
 	    *statusp = -1;
 	    return;
 	}
-	struct stat st;
-	if (stat(filename.c_str(), &st) < 0) {
-	    LOGERR(("Preview: can't stat [%s]\n", filename.c_str()));
-	    QMessageBox::critical(0, "Recoll",
-				  Preview::tr("File does not exist"));
-	    *statusp = -1;
-	    return;
-	}
+
+      // QMessageBox::critical(0, "Recoll", Preview::tr("File does not exist"));
 	
-	FileInterner interner(filename, &st, rclconfig, tmpdir, mtype);
+	FileInterner interner(idoc, rclconfig, tmpdir, 
+                              FileInterner::FIF_forPreview);
+
 	// We don't set the interner's target mtype to html because we
 	// do want the html filter to do its work: we won't use the
 	// text, but we need the conversion to utf-8
 	// interner.setTargetMType("text/html");
 	try {
-	    FileInterner::Status ret = interner.internfile(*out, ipath);
+            string ipath = idoc.ipath;
+	    FileInterner::Status ret = interner.internfile(out, ipath);
 	    if (ret == FileInterner::FIDone || ret == FileInterner::FIAgain) {
 		// FIAgain is actually not nice here. It means that the record
 		// for the *file* of a multidoc was selected. Actually this
 		// shouldn't have had a preview link at all, but we don't know
 		// how to handle it now. Better to show the first doc than
 		// a mysterious error. Happens when the file name matches a
-		// a search term of course.
+		// a search term.
 		*statusp = 0;
+		// If we prefer html and it is available, replace the
+		// text/plain document text
 		if (prefs.previewHtml && !interner.get_html().empty()) {
-		    out->text = interner.get_html();
-		    out->mimetype = "text/html";
+		    out.text = interner.get_html();
+		    out.mimetype = "text/html";
 		}
 	    } else {
-		out->mimetype = interner.getMimetype();
+		out.mimetype = interner.getMimetype();
 		interner.getMissingExternal(missing);
 		*statusp = -1;
 	    }
@@ -706,6 +825,11 @@ class LoadThread : public QThread {
     }
 };
 
+// Insert into editor by chunks so that the top becomes visible
+// earlier for big texts. This provokes some artifacts (adds empty line),
+// so we can't set it too low.
+#define CHUNKL 500*1000
+
 /* A thread to convert to rich text (mark search terms) */
 class ToRichThread : public QThread {
     string &in;
@@ -724,7 +848,7 @@ class ToRichThread : public QThread {
     {
 	DebugLog::getdbl()->setloglevel(loglevel);
 	try {
-	    ptr.plaintorich(in, out, hdata);
+	    ptr.plaintorich(in, out, hdata, CHUNKL);
 	} catch (CancelExcept) {
 	}
     }
@@ -740,10 +864,6 @@ class WaiterThread : public QThread {
     }
 };
 
-#define CHUNKL 50*1000
-#ifndef MIN
-#define MIN(A,B) ((A)<(B)?(A):(B))
-#endif
 class LoadGuard {
     bool *m_bp;
 public:
@@ -751,9 +871,9 @@ public:
     ~LoadGuard() {*m_bp = false; CancelCheck::instance().setCancel(false);}
 };
 
-bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
-				   int docnum)
+bool Preview::loadDocInCurrentTab(const Rcl::Doc &idoc, int docnum)
 {
+    LOGDEB1(("PreviewTextEdit::loadDocInCurrentTab()\n"));
     if (m_loading) {
 	LOGERR(("ALready loading\n"));
 	return false;
@@ -764,18 +884,11 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
 
     m_haveAnchors = false;
 
-    Rcl::Doc doc = idoc;
+    setCurTabProps(idoc, docnum);
 
-    if (doc.meta[Rcl::Doc::keytt].empty()) 
-	doc.meta[Rcl::Doc::keytt] = path_getsimple(doc.url);
-
-    setCurTabProps(fn, doc, docnum);
-
-    char csz[20];
-    sprintf(csz, "%lu", (unsigned long)sz);
     QString msg = QString("Loading: %1 (size %2 bytes)")
-	.arg(QString::fromLocal8Bit(fn.c_str()))
-	.arg(csz);
+	.arg(QString::fromLocal8Bit(idoc.url.c_str()))
+	.arg(QString::fromAscii(idoc.fbytes.c_str()));
 
     // Create progress dialog and aux objects
     const int nsteps = 20;
@@ -783,12 +896,12 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     progress.setMinimumDuration(2000);
     WaiterThread waiter(100);
 
-    // Load and convert file
+    // Load and convert document
+    // idoc came out of the index data (main text and other fields missing). 
+    // foc is the complete one what we are going to extract from storage.
     Rcl::Doc fdoc;
-    // Need to setup config to retrieve possibly local parameters
-    rclconfig->setKeyDir(path_getfather(fn));
     int status = 1;
-    LoadThread lthr(&status, &fdoc, fn, doc.ipath, &doc.mimetype);
+    LoadThread lthr(&status, fdoc, idoc);
     lthr.start();
     int prog;
     for (prog = 1;;prog++) {
@@ -837,14 +950,36 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     // Final text is produced in chunks so that we can display the top
     // while still inserting at bottom
     list<QString> qrichlst;
+    PreviewTextEdit *editor = currentEditor();
+    editor->setText("");
+    editor->setTextFormat(Qt::RichText);
     bool inputishtml = !fdoc.mimetype.compare("text/html");
+
+#if 0
+    // For testing qtextedit bugs...
+    highlightTerms = true;
+    const char *textlist[] =
+    {
+        "Du plain text avec un\n <termtag>termtag</termtag> fin de ligne:",
+        "texte apres le tag\n",
+    };
+    const int listl = sizeof(textlist) / sizeof(char*);
+    for (int i = 0 ; i < listl ; i++)
+        qrichlst.push_back(QString::fromUtf8(textlist[i]));
+#else
     if (highlightTerms) {
+	QStyleSheetItem *item = 
+	    new QStyleSheetItem(editor->styleSheet(), "termtag" );
+	item->setColor(prefs.qtermcolor);
+	item->setFontWeight(QFont::Bold);
 	progress.setLabelText(tr("Creating preview text"));
 	qApp->processEvents();
+
 	if (inputishtml) {
 	    LOGDEB1(("Preview: got html %s\n", fdoc.text.c_str()));
 	    m_plaintorich.set_inputhtml(true);
 	} else {
+	    LOGDEB1(("Preview: got plain %s\n", fdoc.text.c_str()));
 	    m_plaintorich.set_inputhtml(false);
 	}
 	list<string> richlst;
@@ -894,22 +1029,14 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
 	if (inputishtml) {
 	    qrichlst.push_back(qr);
 	} else {
+            editor->setTextFormat(Qt::PlainText);
 	    for (int pos = 0; pos < (int)qr.length(); pos += l) {
 		l = MIN(CHUNKL, qr.length() - pos);
 		qrichlst.push_back(qr.mid(pos, l));
 	    }
 	}
     }
-	    
-    // Load into editor
-    PreviewTextEdit *editor = getCurrentEditor();
-    editor->setText("");
-    if (highlightTerms) {
-	QStyleSheetItem *item = 
-	    new QStyleSheetItem(editor->styleSheet(), "termtag" );
-	item->setColor(prefs.qtermcolor);
-	item->setFontWeight(QFont::Bold);
-    }
+#endif
 
     prog = 2 * nsteps / 3;
     progress.setLabelText(tr("Loading preview text into editor"));
@@ -921,6 +1048,9 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
 	qApp->processEvents();
 
 	editor->append(*it);
+        // We need to save the rich text for printing, the editor does
+        // not do it consistently for us.
+        editor->m_data.richtxt.append(*it);
 
 	// Stay at top
 	if (instep < 5) {
@@ -936,11 +1066,16 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     }
 
     progress.close();
-    TabData *d = tabDataForCurrent();
-    if (d) {
-	fdoc.text.clear();
-	d->fdoc = fdoc;
-    }
+
+    // Maybe the text was actually empty ? Switch to fields then. Else free-up 
+    // the text memory.
+    bool textempty = fdoc.text.empty();
+    if (!textempty)
+        fdoc.text.clear(); 
+    editor->m_data.fdoc = fdoc;
+    if (textempty)
+        editor->toggleFields();
+
     m_haveAnchors = m_plaintorich.lastanchor != 0;
     if (searchTextLine->text().length() != 0) {
 	// If there is a current search string, perform the search
@@ -958,7 +1093,12 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     }
 
     // Enter document in document history
-    g_dynconf->enterDoc(fn, doc.ipath);
+    map<string,string>::const_iterator udit = idoc.meta.find(Rcl::Doc::keyudi);
+    if (udit != idoc.meta.end())
+        historyEnterDoc(g_dynconf, udit->second);
+
+    // Switch format back to plaintext so that selections generate plain text
+    editor->setTextFormat(Qt::PlainText);
 
     editor->setFocus();
     emit(previewExposed(this, m_searchId, docnum));
@@ -966,40 +1106,101 @@ bool Preview::loadFileInCurrentTab(string fn, size_t sz, const Rcl::Doc &idoc,
     return true;
 }
 
-
 RCLPOPUP *PreviewTextEdit::createPopupMenu(const QPoint&)
 {
+    LOGDEB1(("PreviewTextEdit::createPopupMenu()\n"));
     RCLPOPUP *popup = new RCLPOPUP(this);
-    if (m_savedText.isEmpty()) {
-	popup->insertItem(tr("Show fields"), this, SLOT(menuToggleFields()));
+    if (!m_dspflds) {
+	popup->insertItem(tr("Show fields"), this, SLOT(toggleFields()));
     } else {
-	popup->insertItem(tr("Show main text"), this, SLOT(menuToggleFields()));
+	popup->insertItem(tr("Show main text"), this, SLOT(toggleFields()));
     }
+    popup->insertItem(tr("Print"), this, SLOT(print()));
     return popup;
 }
 
-void PreviewTextEdit::menuToggleFields()
+// Either display document fields or main text
+void PreviewTextEdit::toggleFields()
 {
-    if (!m_savedText.isEmpty()) {
-	setText(m_savedText);
-	m_savedText = "";
+    LOGDEB1(("PreviewTextEdit::toggleFields()\n"));
+
+    // If currently displaying fields, switch to body text
+    if (m_dspflds) {
+	setText(m_data.richtxt);
+        m_dspflds = false;
 	return;
     }
-    TabData *d = m_preview->tabDataForCurrent();
-    if (!d) 
-	return;
+
+    // Else display fields
+    m_dspflds = true;
     QString txt = "<html><head></head><body>\n";
-    txt += "<b>" + QString::fromLocal8Bit(d->fn.c_str());
-    if (!d->ipath.empty())
-	txt += "|" + QString::fromUtf8(d->ipath.c_str());
+    txt += "<b>" + QString::fromLocal8Bit(m_data.url.c_str());
+    if (!m_data.ipath.empty())
+	txt += "|" + QString::fromUtf8(m_data.ipath.c_str());
     txt += "</b><br><br>";
     txt += "<dl>\n";
-    for (map<string,string>::const_iterator it = d->fdoc.meta.begin();
-	 it != d->fdoc.meta.end(); it++) {
+    for (map<string,string>::const_iterator it = m_data.fdoc.meta.begin();
+	 it != m_data.fdoc.meta.end(); it++) {
 	txt += "<dt>" + QString::fromUtf8(it->first.c_str()) + "</dt> " 
 	    + "<dd>" + QString::fromUtf8(it->second.c_str()) + "</dd>\n";
     }
     txt += "</dl></body></html>";
-    m_savedText = text();
     setText(txt);
 }
+
+void PreviewTextEdit::print()
+{
+    LOGDEB1(("PreviewTextEdit::print\n"));
+    if (!m_preview)
+        return;
+	
+#ifndef QT_NO_PRINTER
+    QPrinter printer;
+    QPrintDialog *dialog = new QPrintDialog(&printer, this);
+#if (QT_VERSION >= 0x040000)
+    dialog->setWindowTitle(tr("Print Current Preview"));
+#endif
+    if (dialog->exec() != QDialog::Accepted)
+        return;
+
+    // A qt4 version of this would just be :
+    // document()->print(&printer); But as we are using a
+    // q3textedit, we have to do the q3 printing dance, even under
+    // qt4. The following code is taken from
+    // qt3/examples/textdrawing/qtextedit.cpp
+    printer.setFullPage(TRUE);
+    QPaintDeviceMetrics screen( this );
+    printer.setResolution( screen.logicalDpiY() );
+    QPainter p( &printer );
+    QPaintDeviceMetrics metrics( p.device() );
+    int dpix = metrics.logicalDpiX();
+    int dpiy = metrics.logicalDpiY();
+    const int margin = 72; // pt
+    QRect body( margin * dpix / 72, margin * dpiy / 72,
+                metrics.width() - margin * dpix / 72 * 2,
+                metrics.height() - margin * dpiy / 72 * 2 );
+    QFont font( "times", 10 );
+    // Dont want to use text() here, this is the plain text. We 
+    // want the rich text. For some reason we don't need this for fields??
+    const QString &richtxt  = m_dspflds ? text() : m_data.richtxt;
+    QSimpleRichText richText(richtxt, font, this->context(), 
+                             this->styleSheet(),
+                             this->mimeSourceFactory(), body.height() );
+    richText.setWidth( &p, body.width() );
+    QRect view( body );
+    int page = 1;
+    do {
+        richText.draw( &p, body.left(), body.top(), view, colorGroup() );
+        view.moveBy( 0, body.height() );
+        p.translate( 0 , -body.height() );
+        p.setFont( font );
+        p.drawText( view.right() - p.fontMetrics().width( QString::number( page ) ),
+                    view.bottom() + p.fontMetrics().ascent() + 5, QString::number( page ) );
+        if ( view.top()  >= richText.height() )
+            break;
+        printer.newPage();
+        page++;
+    } while (TRUE);
+#endif
+}
+
