@@ -43,6 +43,8 @@ using std::pair;
 #else
 #include <q3filedialog.h>
 #define QFileDialog  Q3FileDialog
+#include <q3toolbar.h>
+#define QToolBar  Q3ToolBar
 #endif
 
 #include <qtabwidget.h>
@@ -68,7 +70,6 @@ using std::pair;
 #include "pathut.h"
 #include "smallut.h"
 #include "advsearch_w.h"
-#include "rclversion.h"
 #include "sortseq.h"
 #include "uiprefs_w.h"
 #include "guiutils.h"
@@ -120,6 +121,12 @@ void RclMain::init()
     spellform = 0;
     m_idxStatusAck = false;
 
+    periodictimer = new QTimer(this);
+
+    // At least some versions of qt4 don't display the status bar if
+    // it's not created here.
+    (void)statusBar();
+
     (void)new HelpClient(this);
     HelpClient::installMap(this->name(), "RCL.SEARCH.SIMPLE");
 
@@ -165,6 +172,16 @@ void RclMain::init()
     }
     preferencesMenu->setItemChecked(curid, true);
 
+    // Toolbar+combobox version of the category selector
+    QComboBox *catgCMB = 0;
+    if (prefs.catgToolBar) {
+	catgCMB = new QComboBox(FALSE, new QToolBar(this), "catCMB");
+	catgCMB->insertItem(tr("All"));
+#if (QT_VERSION >= 0x040000)
+	catgCMB->setToolTip(tr("Document category filter"));
+#endif
+    }
+
     // Document categories buttons
     catgBGRP->setColumnLayout(1, Qt::Vertical);
     list<string> cats;
@@ -178,12 +195,21 @@ void RclMain::init()
 	QString catgnm = QString::fromUtf8(it->c_str(), it->length());
 	m_catgbutvec.push_back(*it);
 	but->setText(tr(catgnm));
+	if (prefs.catgToolBar && catgCMB)
+	    catgCMB->insertItem(tr(catgnm));
     }
     catgBGRP->setButton(0);
-
+    if (prefs.catgToolBar)
+	catgBGRP->hide();
     // Connections
     connect(sSearch, SIGNAL(startSearch(RefCntr<Rcl::SearchData>)), 
 		this, SLOT(startSearch(RefCntr<Rcl::SearchData>)));
+#if QT_VERSION >= 0x040000
+    sSearch->queryText->installEventFilter(this);
+#else
+    sSearch->queryText->lineEdit()->installEventFilter(this);
+#endif
+
     connect(preferencesMenu, SIGNAL(activated(int)),
 	    this, SLOT(setStemLang(int)));
     connect(preferencesMenu, SIGNAL(aboutToShow()),
@@ -232,17 +258,19 @@ void RclMain::init()
 	    this, SLOT(showAdvSearchDialog()));
     connect(toolsSort_parametersAction, SIGNAL(activated()), 
 	    this, SLOT(showSortDialog()));
-#ifdef RCL_USE_ASPELL
     connect(toolsSpellAction, SIGNAL(activated()), 
 	    this, SLOT(showSpellDialog()));
-#else
-    toolsSpellAction->setEnabled(FALSE);
-#endif
 
     connect(indexConfigAction, SIGNAL(activated()), this, SLOT(showIndexConfig()));
     connect(queryPrefsAction, SIGNAL(activated()), this, SLOT(showUIPrefs()));
     connect(extIdxAction, SIGNAL(activated()), this, SLOT(showExtIdxDialog()));
     connect(catgBGRP, SIGNAL(clicked(int)), this, SLOT(catgFilter(int)));
+    if (prefs.catgToolBar && catgCMB)
+	connect(catgCMB, SIGNAL(activated(int)), this, SLOT(catgFilter(int)));
+    connect(periodictimer, SIGNAL(timeout()), this, SLOT(periodic100()));
+    // Start timer on a slow period (used for checking ^C). Will be
+    // speeded up during indexing
+    periodictimer->start(1000);
 
 #if (QT_VERSION < 0x040000)
     nextPageAction->setIconSet(createIconSet("nextpage.png"));
@@ -274,6 +302,54 @@ void RclMain::init()
     }
 }
 
+// This is called by a timer right after we come up. Try to open
+// the database and talk to the user if we can't
+void RclMain::initDbOpen()
+{
+    bool needindexconfig = false;
+    bool nodb = false;
+    string reason;
+    if (!maybeOpenDb(reason)) {
+        nodb = true;
+	switch (QMessageBox::
+#if (QT_VERSION >= 0x030200)
+		question
+#else
+		information
+#endif
+		(this, "Recoll",
+		 qApp->translate("Main", "Could not open database in ") +
+		 QString::fromLocal8Bit(rclconfig->getDbDir().c_str()) +
+		 qApp->translate("Main", 
+			       ".\n"
+			       "Click Cancel if you want to edit the configuration file before indexation starts, or Ok to let it proceed."),
+		 "Ok", "Cancel", 0,   0)) {
+
+	case 0: // Ok: indexing is going to start.
+	    start_indexing(true);
+	    break;
+
+	case 1: // Cancel
+	    needindexconfig = true;
+	    break;
+	}
+    }
+
+    if (needindexconfig) {
+	startIndexingAfterConfig = 1;
+	showIndexConfig();
+    } else {
+	if (prefs.startWithAdvSearchOpen)
+	    showAdvSearchDialog();
+	if (prefs.startWithSortToolOpen)
+	    showSortDialog();
+        // If we have something in the search entry, it comes from a
+        // command line argument
+        if (!nodb && sSearch->hasSearchString())
+            QTimer::singleShot(0, sSearch, SLOT(startSimpleSearch()));
+    }
+}
+
 void RclMain::setStemLang(int id)
 {
     LOGDEB(("RclMain::setStemLang(%d)\n", id));
@@ -364,12 +440,16 @@ void RclMain::fileExit()
     prefs.ssearchTyp = sSearch->searchTypCMB->currentItem();
     if (asearchform)
 	delete asearchform;
-    // Let the exit handler clean up things
+    // We'd prefer to do this in the exit handler, but it's apparently to late
+    // and some of the qt stuff is already dead at this point?
+    LOGDEB2(("RclMain::fileExit() : stopping idx thread\n"));
+    stop_idxthread();
+    // Let the exit handler clean up the rest (internal recoll stuff).
     exit(0);
 }
 
-// This is called on a 100ms timer checks the status of the indexing
-// thread and a possible need to exit
+// This is called by a periodic timer to check the status of the
+// indexing thread and a possible need to exit
 void RclMain::periodic100()
 {
     static int toggle = 0;
@@ -396,14 +476,18 @@ void RclMain::periodic100()
 	    // looking at them either). Fixing this would be
 	    // relatively complicated (keep an open/close gen number
 	    // and check this / restart query in DocSeqDb() ?)
-	    rcldb->close();
+	    string reason;
+	    maybeOpenDb(reason, 1);
+            periodictimer->changeInterval(1000);
 	}
     } else {
 	// Indexing is running
 	m_idxStatusAck = false;
 	fileToggleIndexingAction->setText(tr("Stop &Indexing"));
+	fileToggleIndexingAction->setEnabled(TRUE);
+        periodictimer->changeInterval(100);
 	// The toggle thing is for the status to flash
-	if (toggle == 0) {
+	if (toggle < 9) {
 	    QString msg = tr("Indexing in progress: ");
 	    DbIxStatus status = idxthread_idxStatus();
 	    QString phs;
@@ -440,25 +524,24 @@ void RclMain::periodic100()
 	fileExit();
 }
 
+// This gets called when the indexing action is activated. It starts
+// the requested action, and disables the menu entry. This will be
+// re-enabled by the indexing status check
 void RclMain::toggleIndexing()
 {
     if (idxthread_getStatus() == IDXTS_NULL) {
-	// Indexing in progress
+	// Indexing was in progress, stop it
 	stop_indexing();
+        periodictimer->changeInterval(1000);
 	fileToggleIndexingAction->setText(tr("Update &Index"));
     } else {
 	start_indexing(false);
+        periodictimer->changeInterval(100);
 	fileToggleIndexingAction->setText(tr("Stop &Indexing"));
     }
     fileToggleIndexingAction->setEnabled(FALSE);
 }
 
-// Note that all our 'urls' are like : file://...
-static string urltolocalpath(string url)
-{
-    return url.substr(7, string::npos);
-}
-
 // Start a db query and set the reslist docsource
 void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
 {
@@ -494,6 +577,9 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata)
     DocSequenceDb *src = 
 	new DocSequenceDb(RefCntr<Rcl::Query>(query), 
 			  string(tr("Query results").utf8()), sdata);
+    src->setAbstractParams(prefs.queryBuildAbstract, 
+                           prefs.queryReplaceAbstract);
+
     resList->setDocSource(RefCntr<DocSequence>(src));
     QApplication::restoreOverrideCursor();
 }
@@ -591,8 +677,9 @@ void RclMain::showExtIdxDialog()
 
 void RclMain::showAboutDialog()
 {
-    string vstring = string("Recoll ") + rclversion + 
-	"<br>" + "http://www.recoll.org";
+    string vstring = Rcl::version_string() +
+        string("<br> http://www.recoll.org") +
+	string("<br> http://www.xapian.org");
     QMessageBox::information(this, tr("About Recoll"), vstring.c_str());
 }
 void RclMain::showMissingHelpers()
@@ -639,14 +726,6 @@ void RclMain::startPreview(int docnum, int mod)
 	return;
     }
 	
-    // Check file exists in file system
-    string fn = urltolocalpath(doc.url);
-    struct stat st;
-    if (stat(fn.c_str(), &st) < 0) {
-	QMessageBox::warning(0, "Recoll", tr("Cannot access document file: ") +
-			     fn.c_str());
-	return;
-    }
     if (mod & Qt::ShiftButton) {
 	// User wants new preview window
 	curPreview = 0;
@@ -675,7 +754,7 @@ void RclMain::startPreview(int docnum, int mod)
 	curPreview->setCaption(resList->getDescription());
 	curPreview->show();
     } 
-    curPreview->makeDocCurrent(fn, st.st_size, doc, docnum);
+    curPreview->makeDocCurrent(doc, docnum);
 }
 
 /** 
@@ -687,14 +766,6 @@ void RclMain::startPreview(int docnum, int mod)
  */
 void RclMain::startPreview(Rcl::Doc doc)
 {
-    // Check file exists in file system
-    string fn = urltolocalpath(doc.url);
-    struct stat st;
-    if (stat(fn.c_str(), &st) < 0) {
-	QMessageBox::warning(0, "Recoll", tr("Cannot access document file: ") +
-			     fn.c_str());
-	return;
-    }
     Preview *preview = new Preview(0, HiliteData());
     if (preview == 0) {
 	QMessageBox::warning(0, tr("Warning"), 
@@ -706,7 +777,7 @@ void RclMain::startPreview(Rcl::Doc doc)
     connect(preview, SIGNAL(wordSelect(QString)),
 	    this, SLOT(ssearchAddTerm(QString)));
     preview->show();
-    preview->makeDocCurrent(fn, st.st_size, doc, 0);
+    preview->makeDocCurrent(doc, 0);
 }
 
 // Show next document from result list in current preview tab
@@ -752,16 +823,7 @@ void RclMain::previewPrevOrNextInTab(Preview * w, int sid, int docnum, bool nxt)
 	return;
     }
 	
-    // Check that file exists in file system
-    string fn = urltolocalpath(doc.url);
-    struct stat st;
-    if (stat(fn.c_str(), &st) < 0) {
-	QMessageBox::warning(0, "Recoll", tr("Cannot access document file: ") +
-			     fn.c_str());
-	return;
-    }
-
-    w->makeDocCurrent(fn, st.st_size, doc, docnum, true);
+    w->makeDocCurrent(doc, docnum, true);
 }
 
 // Preview tab exposed: if the preview comes from the currently
@@ -813,7 +875,6 @@ void RclMain::saveDocToFile(int docnum)
 				" from database"));
 	return;
     }
-    string fn = urltolocalpath(doc.url);
     QString s = 
 	QFileDialog::getSaveFileName(path_home().c_str(),
 				     "",  this,
@@ -821,8 +882,7 @@ void RclMain::saveDocToFile(int docnum)
 				     tr("Choose a file name to save under"));
     string tofile((const char *)s.local8Bit());
     TempFile temp; // not used
-    if (!FileInterner::idocToFile(temp, tofile, rclconfig, fn, 
-				  doc.ipath, doc.mimetype)) {
+    if (!FileInterner::idocToFile(temp, tofile, rclconfig, doc)) {
 	QMessageBox::warning(0, "Recoll",
 			     tr("Cannot extract document or create "
 				"temporary file"));
@@ -864,22 +924,44 @@ void RclMain::startNativeViewer(int docnum)
     startNativeViewer(doc);
 }
 
+// Convert to file path if url is like file://
+static string fileurltolocalpath(string url)
+{
+    if (url.find("file://") == 0)
+        return url.substr(7, string::npos);
+    return string();
+}
+
 void RclMain::startNativeViewer(Rcl::Doc doc)
 {
     // Look for appropriate viewer
-    string cmd;
+    string cmdplusattr;
     if (prefs.useDesktopOpen) {
-	cmd = rclconfig->getMimeViewerDef("application/x-all");
+	cmdplusattr = rclconfig->getMimeViewerDef("application/x-all", "");
     } else {
-	cmd = rclconfig->getMimeViewerDef(doc.mimetype);
+        string apptag;
+        map<string,string>::const_iterator it;
+        if ((it = doc.meta.find(Rcl::Doc::keyapptg)) != doc.meta.end())
+            apptag = it->second;
+	cmdplusattr = rclconfig->getMimeViewerDef(doc.mimetype, apptag);
     }
 
-    if (cmd.length() == 0) {
+    if (cmdplusattr.length() == 0) {
 	QMessageBox::warning(0, "Recoll", 
-			     tr("No external viewer configured for mime type ")
-			     + doc.mimetype.c_str());
+			     tr("No external viewer configured for mime type [")
+			     + doc.mimetype.c_str() + "]");
 	return;
     }
+
+    // Extract possible viewer attributes
+    ConfSimple attrs;
+    string cmd;
+    rclconfig->valueSplitAttributes(cmdplusattr, cmd, attrs);
+    bool ignoreipath = false;
+    if (attrs.get("ignoreipath", cmdplusattr))
+        ignoreipath = stringToBool(cmdplusattr);
+
+    // Split the command line
     list<string> lcmd;
     if (!stringToStrings(cmd, lcmd)) {
 	QMessageBox::warning(0, "Recoll", 
@@ -890,6 +972,8 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 	return;
     }
 
+    // Look for the command to execute in the exec path and a few
+    // other places
     string cmdpath;
     if (prefs.useDesktopOpen) {
 	// Findfilter searches the recoll filters directory in
@@ -912,9 +996,11 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 	    cmd = cmdpath + " %u";
 	}
     }
+
+    // Command not found: start the user dialog to help find another one:
     if (cmdpath.empty()) {
 	QString mt = QString::fromAscii(doc.mimetype.c_str());
-	QString message = tr("The viewer specified in mimeconf for %1: %2"
+	QString message = tr("The viewer specified in mimeview for %1: %2"
 			     " is not found.\nDo you want to start the "
 			     " preferences dialog ?")
 	    .arg(mt).arg(QString::fromLocal8Bit(lcmd.front().c_str()));
@@ -929,24 +1015,25 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 	case 1:
 	    break;
 	}
+        // The user will have to click on the link again to try the
+        // new command.
 	return;
     }
 
-    // For files with an ipath, we do things differently depending if the 
-    // configured command seems to be able to grok it or not.
-    bool wantsipath = cmd.find("%i") != string::npos;
+    // We may need a temp file, or not depending on the command arguments
+    // and the fact that this is a subdoc or not.
+    bool wantsipath = (cmd.find("%i") != string::npos) || ignoreipath;
+    bool wantsfile = cmd.find("%f") != string::npos;
     bool istempfile = false;
-    string fn = urltolocalpath(doc.url);
-    string url;
+    string fn = fileurltolocalpath(doc.url);
+    string url = doc.url;
+
+    // If the command wants a file but this is not a file url, or
+    // there is an ipath that it won't understand, we need a temp file:
     rclconfig->setKeyDir(path_getfather(fn));
-    if (doc.ipath.empty() || wantsipath) {
-	url = doc.url;
-    } else {
-	// There is an ipath and the command does not know about
-	// them. We need a temp file.
+    if ((wantsfile && fn.empty()) || (!wantsipath && !doc.ipath.empty())) {
 	TempFile temp;
-	if (!FileInterner::idocToFile(temp, string(), rclconfig, fn, 
-					doc.ipath, doc.mimetype)) {
+	if (!FileInterner::idocToFile(temp, string(), rclconfig, doc)) {
 	    QMessageBox::warning(0, "Recoll",
 				 tr("Cannot extract document or create "
 				    "temporary file"));
@@ -959,11 +1046,28 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
     }
 
     // Substitute %xx inside prototype command
+    string efftime;
+    if (!doc.dmtime.empty() || !doc.fmtime.empty()) {
+        efftime = doc.dmtime.empty() ? doc.fmtime : doc.dmtime;
+    } else {
+        efftime = "0";
+    }
+    // Try to keep the letters used more or less consistent with the reslist
+    // paragraph format.
     string ncmd;
-    map<char, string> subs;
-    subs['u'] = escapeShell(url);
-    subs['f'] = escapeShell(fn);
-    subs['i'] = escapeShell(doc.ipath);
+    map<string, string> subs;
+    subs["D"] = escapeShell(efftime);
+    subs["f"] = escapeShell(fn);
+    subs["i"] = escapeShell(doc.ipath);
+    subs["M"] = escapeShell(doc.mimetype);
+    subs["U"] = escapeShell(url);
+    subs["u"] = escapeShell(url);
+    // Let %(xx) access all metadata.
+    for (map<string,string>::const_iterator it = doc.meta.begin();
+         it != doc.meta.end(); it++) {
+        subs[it->first] = escapeShell(it->second);
+    }
+
     pcSubst(cmd, ncmd, subs);
 
     ncmd += " &";
@@ -976,8 +1080,9 @@ void RclMain::startNativeViewer(Rcl::Doc doc)
 	    QString::fromUtf8(prcmd.c_str()) + "]";
 	stb->message(msg, 5000);
     }
+
     if (!istempfile)
-	g_dynconf->enterDoc(fn, doc.ipath);
+	historyEnterDoc(g_dynconf, doc.meta[Rcl::Doc::keyudi]);
     // We should actually monitor these processes so that we can
     // delete the temp files when they exit
     LOGDEB(("Executing: [%s]\n", ncmd.c_str()));
@@ -1063,7 +1168,7 @@ void RclMain::eraseDocHistory()
 {
     // Clear file storage
     if (g_dynconf)
-	g_dynconf->eraseAll(RclHistory::docSubkey);
+	g_dynconf->eraseAll(docHistSubKey);
     // Clear possibly displayed history
     if (resList->displayingHistory()) {
 	showDocHistory();
@@ -1117,3 +1222,21 @@ void RclMain::catgFilter(int id)
     resList->setDocSource();
 }
 
+bool RclMain::eventFilter(QObject *, QEvent *event)
+{
+    if (event->type() == QEvent::KeyPress)  {
+        LOGDEB2(("RclMain::eventFilter: keypress\n"));
+	QKeyEvent *ke = (QKeyEvent *)event;
+	if (ke->key() == Qt::Key_PageDown || ke->key() == Qt::Key_PageUp ) {
+            // Page up down are sent to the resList where they make sense
+            QApplication::sendEvent(resList, event);
+            return true;
+        } else if (ke->key() == Qt::Key_Home &&
+                   (ke->state() & Qt::ShiftButton)) {
+            // Shift-Home -> first page of results
+            resList->resultPageFirst();
+            return true;
+        }
+    }
+    return false;
+}
