@@ -34,8 +34,13 @@
 #if QT_VERSION < 0x040000
 #include "rclmain.h"
 #else
+#if QT_VERSION >= 0x040601
+// 4.6.1 uic is broken
+#include "ui_rclmain.h-4.5"
+#else
 #include "ui_rclmain.h"
 #endif
+#endif
 
 //MOC_SKIP_BEGIN
 #if QT_VERSION < 0x040000
