@@ -2,6 +2,8 @@
 static char rcsid[] = "@(#$Id: reslist.cpp,v 1.52 2008-12-17 15:12:08 dockes Exp $ (C) 2005 J.F.Dockes";
 #endif
 
+#include "autoconfig.h"
+
 #include <time.h>
 #include <stdlib.h>
 
@@ -46,11 +48,34 @@ static char rcsid[] = "@(#$Id: reslist.cpp,v 1.52 2008-12-17 15:12:08 dockes Exp
 #include "reslist.h"
 #include "moc_reslist.cpp"
 #include "rclhelp.h"
+#ifdef RCL_USE_ASPELL
+#include "rclaspell.h"
+#endif
 
 #ifndef MIN
 #define MIN(A,B) ((A) < (B) ? (A) : (B))
 #endif
 
+class QtGuiResListPager : public ResListPager {
+public:
+    QtGuiResListPager(ResList *p, int ps) 
+	: ResListPager(ps), m_parent(p) 
+    {}
+    virtual bool append(const string& data);
+    virtual bool append(const string& data, int idx, const Rcl::Doc& doc);
+    virtual string trans(const string& in);
+    virtual string detailsLink();
+    virtual const string &parFormat();
+    virtual string nextUrl();
+    virtual string prevUrl();
+    virtual string pageTop();
+    virtual string iconPath(const string& mt);
+    virtual void suggest(const vector<string>uterms, vector<string>&sugg);
+private:
+    ResList *m_parent;
+};
+
+
 class PlainToRichQtReslist : public PlainToRich {
 public:
     virtual ~PlainToRichQtReslist() {}
@@ -91,6 +116,23 @@ ResList::ResList(QWidget* parent, const char* name)
 
 ResList::~ResList()
 {
+    // These have to exist somewhere for translations to work
+#ifdef __GNUC__
+    __attribute__((unused))
+#endif
+    static const char* strings[] = {
+	QT_TR_NOOP("<p><b>No results found</b><br>"),
+	QT_TR_NOOP("Documents <b>%d-%d</b> out of at least <b>%d</b> for "),
+	QT_TR_NOOP("Documents <b>%d-%d</b> for "),
+	QT_TR_NOOP("Previous"),
+	QT_TR_NOOP("Next"),
+	QT_TR_NOOP("Unavailable document"),
+	QT_TR_NOOP("Preview"),
+	QT_TR_NOOP("Open"),
+	QT_TR_NOOP("(show query)"),
+        QT_TR_NOOP("<p><i>Alternate spellings (accents suppressed): </i>"),
+    };
+
 }
 
 int ResList::newListId()
@@ -141,7 +183,7 @@ void ResList::setDocSource()
     resultPageNext();
 }
 
-void ResList::setSortParams(const DocSeqSortSpec& spec)
+void ResList::setSortParams(DocSeqSortSpec spec)
 {
     LOGDEB(("ResList::setSortParams\n"));
     m_sortspecs = spec;
@@ -346,18 +388,6 @@ void ResList::resultPageFirst()
 
 void ResList::append(const QString &text)
 {
-    // These has to go somewhere for translations to work
-    static const char* strings[] = {
-	QT_TR_NOOP("<p><b>No results found</b><br>"),
-	QT_TR_NOOP("Documents <b>%d-%d</b> out of at least <b>%d</b> for "),
-	QT_TR_NOOP("Documents <b>%d-%d</b> for "),
-	QT_TR_NOOP("Previous"),
-	QT_TR_NOOP("Next"),
-	QT_TR_NOOP("Unavailable document"),
-	QT_TR_NOOP("Preview"),
-	QT_TR_NOOP("Open"),
-	QT_TR_NOOP("(show query)"),
-    };
     QTEXTBROWSER::append(text);
 #if 0
     {
@@ -421,6 +451,39 @@ string QtGuiResListPager::iconPath(const string& mtype)
     return iconpath;
 }
 
+void QtGuiResListPager::suggest(const vector<string>uterms, vector<string>&sugg)
+{
+    sugg.clear();
+#ifdef RCL_USE_ASPELL
+    bool noaspell = false;
+    rclconfig->getConfParam("noaspell", &noaspell);
+    if (noaspell)
+        return;
+    if (!aspell) {
+        LOGERR(("QtGuiResListPager:: aspell not initialized\n"));
+        return;
+    }
+    for (vector<string>::const_iterator uit = uterms.begin();
+         uit != uterms.end(); uit++) {
+        list<string> asuggs;
+        string reason;
+        if (aspell->check(*rcldb, *uit, reason))
+            continue;
+        else if (!reason.empty())
+            return;
+        if (!aspell->suggest(*rcldb, *uit, asuggs, reason)) {
+            LOGERR(("QtGuiResListPager::suggest: aspell failed: %s\n", 
+                    reason.c_str()));
+            continue;
+        }
+        if (!asuggs.empty()) {
+            sugg.push_back(*asuggs.begin());
+        }
+    }
+#endif
+
+}
+
 // Fill up result list window with next screen of hits
 void ResList::resultPageNext()
 {
@@ -538,8 +601,23 @@ void ResList::linkWasClicked(const QString &s, int clkmod)
 
 RCLPOPUP *ResList::createPopupMenu(const QPoint& pos)
 {
+    LOGDEB2(("ResList::createPopupMenu(%d, %d)\n", pos.x(), pos.y()));
     int para = paragraphAt(pos);
-    clicked(para, 0);
+    LOGDEB2(("ResList::createPopupMenu(): para %d\n", para));
+    if (para == 0) {
+        // There is a bug in qt3 paragraphAt() when the click is inside
+        // a table.  paragraphAt() calls textcursor::place() which
+        // fails because of a positioning problem inside the paragraph
+        // (the paragraph is found but place() returns an error
+        // anyway). Try to find the paragraph myself:
+        for (;para < paragraphs(); para++) {
+            QRect rect = paragraphRect(para);
+            if (pos.y() >= rect.y() && pos.y() < rect.y() + rect.height())
+                break;
+        }
+        if (para == paragraphs())
+            para = 0;
+    }
     m_popDoc = docnumfromparnum(para);
     if (m_popDoc < 0) 
 	return 0;
@@ -550,12 +628,14 @@ RCLPOPUP *ResList::createPopupMenu(const QPoint& pos)
     popup->insertItem(tr("Copy &URL"), this, SLOT(menuCopyURL()));
     Rcl::Doc doc;
     if (getDoc(m_popDoc, doc) && !doc.ipath.empty()) {
-	popup->insertItem(tr("Save to File"), this, SLOT(menuSaveToFile()));
+	popup->insertItem(tr("&Write to File"), this, SLOT(menuSaveToFile()));
     }
 
     popup->insertItem(tr("Find &similar documents"), this, SLOT(menuExpand()));
-    popup->insertItem(tr("P&arent document/folder"), 
-		      this, SLOT(menuSeeParent()));
+    popup->insertItem(tr("Preview P&arent document/folder"), 
+		      this, SLOT(menuPreviewParent()));
+    popup->insertItem(tr("&Open Parent document/folder"), 
+		      this, SLOT(menuOpenParent()));
     return popup;
 }
 
@@ -568,20 +648,37 @@ void ResList::menuSaveToFile()
     emit docSaveToFileClicked(m_popDoc);
 }
 
-void ResList::menuSeeParent()
+void ResList::menuPreviewParent()
 {
     Rcl::Doc doc;
-    if (!getDoc(m_popDoc, doc)) 
+    if (!getDoc(m_popDoc, doc) || m_baseDocSource.isNull()) 
 	return;
-    Rcl::Doc doc1;
-    if (FileInterner::getEnclosing(doc.url, doc.ipath, doc1.url, doc1.ipath)) {
-	emit previewRequested(doc1);
+    Rcl::Doc pdoc;
+    if (m_baseDocSource->getEnclosing(doc, pdoc)) {
+	emit previewRequested(pdoc);
     } else {
 	// No parent doc: show enclosing folder with app configured for
 	// directories
-	doc1.url = path_getfather(doc.url);
-	doc1.mimetype = "application/x-fsdirectory";
-	emit editRequested(doc1);
+	pdoc.url = path_getfather(doc.url);
+	pdoc.mimetype = "application/x-fsdirectory";
+	emit editRequested(pdoc);
+    }
+}
+
+void ResList::menuOpenParent()
+{
+    Rcl::Doc doc;
+    if (!getDoc(m_popDoc, doc) || m_baseDocSource.isNull()) 
+	return;
+    Rcl::Doc pdoc;
+    if (m_baseDocSource->getEnclosing(doc, pdoc)) {
+	emit editRequested(pdoc);
+    } else {
+	// No parent doc: show enclosing folder with app configured for
+	// directories
+	pdoc.url = path_getfather(doc.url);
+	pdoc.mimetype = "application/x-fsdirectory";
+	emit editRequested(pdoc);
     }
 }
 
