@@ -28,24 +28,7 @@ class Q3PopupMenu;
 #include "reslistpager.h"
 
 class ResList;
-
-class QtGuiResListPager : public ResListPager {
-public:
-    QtGuiResListPager(ResList *p, int ps) 
-	: ResListPager(ps), m_parent(p) 
-    {}
-    virtual bool append(const string& data);
-    virtual bool append(const string& data, int idx, const Rcl::Doc& doc);
-    virtual string trans(const string& in);
-    virtual string detailsLink();
-    virtual const string &parFormat();
-    virtual string nextUrl();
-    virtual string prevUrl();
-    virtual string pageTop();
-    virtual string iconPath(const string& mt);
-private:
-    ResList *m_parent;
-};
+class QtGuiResListPager;
 
 /**
  * Display a list of document records. The data can be out of the history 
@@ -95,13 +78,14 @@ class ResList : public QTEXTBROWSER
     virtual void menuCopyFN();
     virtual void menuCopyURL();
     virtual void menuExpand();
-    virtual void menuSeeParent();
+    virtual void menuPreviewParent();
+    virtual void menuOpenParent();
     virtual void previewExposed(int);
     virtual void append(const QString &text);
     // Only used for qt ver >=4 but seems we cant undef it
     virtual void selecChanged();
     virtual void setDocSource();
-    virtual void setSortParams(const DocSeqSortSpec &spec);
+    virtual void setSortParams(DocSeqSortSpec spec);
     virtual void setFilterParams(const DocSeqFiltSpec &spec);
 
  signals:
