@@ -191,11 +191,17 @@ void SSearch::startSimpleSearch()
     RefCntr<Rcl::SearchData> rsdata(sdata);
     emit startSearch(rsdata);
 }
+
 void SSearch::setSearchString(const QString& txt)
 {
     queryText->setEditText(txt);
 }
 
+bool SSearch::hasSearchString()
+{
+    return !queryText->lineEdit()->text().isEmpty();
+}
+
 void SSearch::setAnyTermMode()
 {
     searchTypCMB->setCurrentItem(SST_ANY);
