@@ -87,6 +87,8 @@ void UIPrefsDialog::init()
     connect(buttonCancel, SIGNAL(clicked()), this, SLOT(reject()));
     connect(buildAbsCB, SIGNAL(toggled(bool)), 
 	    replAbsCB, SLOT(setEnabled(bool)));
+    connect(useDesktopOpenCB, SIGNAL(toggled(bool)), 
+	    viewActionPB, SLOT(setDisabled(bool)));
 
     setFromPrefs();
 }
@@ -98,13 +100,18 @@ void UIPrefsDialog::setFromPrefs()
     pageLenSB->setValue(prefs.respagesize);
     collapseDupsCB->setChecked(prefs.collapseDuplicates);
     maxHLTSB->setValue(prefs.maxhltextmbs);
+    catgToolBarCB->setChecked(prefs.catgToolBar);
     autoSearchCB->setChecked(prefs.autoSearchOnWS);
     syntlenSB->setValue(prefs.syntAbsLen);
     syntctxSB->setValue(prefs.syntAbsCtx);
 
     initStartAdvCB->setChecked(prefs.startWithAdvSearchOpen);
     initStartSortCB->setChecked(prefs.startWithSortToolOpen);
+
+    // External editor. Can use desktop prefs or internal
     useDesktopOpenCB->setChecked(prefs.useDesktopOpen);
+    viewActionPB->setEnabled(!prefs.useDesktopOpen);
+
     keepSortCB->setChecked(prefs.keepSort);
     previewHtmlCB->setChecked(prefs.previewHtml);
     // Query terms color
@@ -171,6 +178,7 @@ void UIPrefsDialog::setFromPrefs()
 void UIPrefsDialog::accept()
 {
     prefs.autoSearchOnWS = autoSearchCB->isChecked();
+    prefs.catgToolBar = catgToolBarCB->isChecked();
     prefs.respagesize = pageLenSB->value();
     prefs.collapseDuplicates = collapseDupsCB->isChecked();
     prefs.maxhltextmbs = maxHLTSB->value();
