@@ -23,11 +23,14 @@ static char rcsid[] = "@(#$Id: docseqdb.cpp,v 1.9 2008-11-13 10:57:46 dockes Exp
 #include "docseqdb.h"
 #include "rcldb.h"
 #include "debuglog.h"
+#include "internfile.h"
 
 DocSequenceDb::DocSequenceDb(RefCntr<Rcl::Query> q, const string &t, 
 			     RefCntr<Rcl::SearchData> sdata) 
     : DocSequence(t), m_q(q), m_sdata(sdata), m_fsdata(sdata),
-      m_rescnt(-1), m_filt(false)
+      m_rescnt(-1), m_filt(false),
+      m_queryBuildAbstract(true),
+      m_queryReplaceAbstract(false)
 {
 }
 
@@ -42,6 +45,11 @@ bool DocSequenceDb::getTerms(vector<string>& terms,
     return m_fsdata->getTerms(terms, groups, gslks);
 }
 
+void DocSequenceDb::getUTerms(vector<string>& terms)
+{
+    m_sdata->getUTerms(terms);
+}
+
 string DocSequenceDb::getDescription() 
 {
     return m_fsdata->getDescription();
@@ -66,10 +74,25 @@ string DocSequenceDb::getAbstract(Rcl::Doc &doc)
     if (!m_q->whatDb())
 	return doc.meta[Rcl::Doc::keyabs];
     string abstract;
-    m_q->whatDb()->makeDocAbstract(doc, m_q.getptr(), abstract);
+
+     if (m_queryBuildAbstract && (doc.syntabs || m_queryReplaceAbstract)) {
+        m_q->whatDb()->makeDocAbstract(doc, m_q.getptr(), abstract);
+    } else {
+        abstract = doc.meta[Rcl::Doc::keyabs];
+    }
+
     return abstract.empty() ? doc.meta[Rcl::Doc::keyabs] : abstract;
 }
 
+bool DocSequenceDb::getEnclosing(Rcl::Doc& doc, Rcl::Doc& pdoc) 
+{
+    string udi;
+    if (!FileInterner::getEnclosing(doc.url, doc.ipath, pdoc.url, pdoc.ipath,
+                                    udi))
+        return false;
+    return m_q->whatDb()->getDoc(udi, pdoc);
+}
+
 list<string> DocSequenceDb::expand(Rcl::Doc &doc)
 {
     return m_q->expand(doc);
