@@ -98,6 +98,22 @@ void ResListPager::displayPage()
 
     if (pageEmpty()) {
 	chunk += trans("<p><b>No results found</b><br>");
+        vector<string>uterms;
+        m_docSource->getUTerms(uterms);
+        if (!uterms.empty()) {
+            vector<string> spellings;
+            suggest(uterms, spellings);
+            if (!spellings.empty()) {
+                chunk += 
+                 trans("<p><i>Alternate spellings (accents suppressed): </i>");
+                for (vector<string>::iterator it = spellings.begin();
+                     it != spellings.end(); it++) {
+                    chunk += *it;
+                    chunk += " ";
+                }
+                chunk += "</p>";
+            }
+        }
     } else {
 	unsigned int resCnt = m_docSource->getResCnt();
 	if (m_winfirst + m_respage.size() < resCnt) {
@@ -201,12 +217,8 @@ void ResListPager::displayPage()
 	    sizebuf = displayableBytes(fsize);
 	}
 
-	string abstract;
-	if (m_queryBuildAbstract && (doc.syntabs || m_queryReplaceAbstract)) {
-	    abstract = m_docSource->getAbstract(doc);
-	} else {
-	    abstract = doc.meta[Rcl::Doc::keyabs];
-	}
+	string abstract = m_docSource->getAbstract(doc);
+
 	// No need to call escapeHtml(), plaintorich handles it
 	list<string> lr;
 	m_hiliter->set_inputhtml(false);
@@ -221,7 +233,13 @@ void ResListPager::displayPage()
 	    linksbuf += string("<a href=") + vlbuf + ">" + trans("Preview") 
 		+ "</a>&nbsp;&nbsp;";
 	}
-	if (!RclConfig::getMainConfig()->getMimeViewerDef(doc.mimetype).empty()) {
+
+        string apptag;
+        map<string,string>::const_iterator it;
+        if ((it = doc.meta.find(Rcl::Doc::keyapptg)) != doc.meta.end())
+            apptag = it->second;
+
+	if (!RclConfig::getMainConfig()->getMimeViewerDef(doc.mimetype, apptag).empty()) {
 	    sprintf(vlbuf, "E%d", docnumforlinks);
 	    linksbuf += string("<a href=") + vlbuf + ">" + trans("Open")
 		+ "</a>";
@@ -237,20 +255,23 @@ void ResListPager::displayPage()
 	    chunk += "<p>";
 
 	// Configurable stuff
-	map<char,string> subs;
-	subs['A'] = !richabst.empty() ? richabst + "<br>" : "";
-	subs['D'] = datebuf;
-	subs['I'] = iconpath;
-	subs['i'] = doc.ipath;
-	subs['K'] = !doc.meta[Rcl::Doc::keykw].empty() ? 
+	map<string,string> subs;
+	subs["A"] = !richabst.empty() ? richabst + "<br>" : "";
+	subs["D"] = datebuf;
+	subs["I"] = iconpath;
+	subs["i"] = doc.ipath;
+	subs["K"] = !doc.meta[Rcl::Doc::keykw].empty() ? 
 	    escapeHtml(doc.meta[Rcl::Doc::keykw]) + "<br>" : "";
-	subs['L'] = linksbuf;
-	subs['N'] = numbuf;
-	subs['M'] = doc.mimetype;
-	subs['R'] = perbuf;
-	subs['S'] = sizebuf;
-	subs['T'] = escapeHtml(doc.meta[Rcl::Doc::keytt]);
-	subs['U'] = url;
+	subs["L"] = linksbuf;
+	subs["N"] = numbuf;
+	subs["M"] = doc.mimetype;
+	subs["R"] = perbuf;
+	subs["S"] = sizebuf;
+	subs["T"] = escapeHtml(doc.meta[Rcl::Doc::keytt]);
+	subs["U"] = url;
+
+        // Let %(xx) access all metadata.
+        subs.insert(doc.meta.begin(), doc.meta.end());
 
 	string formatted;
 	pcSubst(parFormat(), formatted, subs);
@@ -281,10 +302,13 @@ void ResListPager::displayPage()
     append(chunk);
 }
 
+// Default implementations for things that should be implemented by 
+// specializations
 string ResListPager::nextUrl()
 {
     return "n-1";
 }
+
 string ResListPager::prevUrl()
 {
     return "p-1";
@@ -298,7 +322,6 @@ string ResListPager::iconPath(const string& mtype)
     return iconpath;
 }
 
-// Default implementations for things that should be re-implemented by our user.
 bool ResListPager::append(const string& data)
 {
     fprintf(stderr, "%s", data.c_str());
