@@ -33,6 +33,8 @@ static char rcsid[] = "@(#$Id: rcldb.cpp,v 1.154 2008-12-17 16:19:58 dockes Exp
 using namespace std;
 #endif /* NO_NAMESPACES */
 
+#include "xapian/version.h"
+
 #include "rclconfig.h"
 #include "rcldb.h"
 #include "rcldb_p.h"
@@ -49,6 +51,7 @@ using namespace std;
 #include "rclquery.h"
 #include "rclquery_p.h"
 #include "md5.h"
+#include "rclversion.h"
 
 #ifndef MAX
 #define MAX(A,B) (A>B?A:B)
@@ -70,6 +73,11 @@ static const unsigned int baseTextPosition = 100000;
 namespace Rcl {
 #endif
 
+string version_string(){
+    return string("Recoll ") + string(rclversionstr) + string(" + Xapian ") +
+        string(Xapian::version_string());
+}
+
 // Synthetic abstract marker (to discriminate from abstract actually
 // found in document)
 const static string rclSyntAbs("?!#@");
@@ -83,7 +91,7 @@ static inline string make_uniterm(const string& udi)
     return uniterm;
 }
 // Compute parent term used to link documents to their parent document (if any)
-// "" + parent external udi
+// "F" + parent external udi
 static inline string make_parentterm(const string& udi)
 {
     // I prefer to be in possible conflict with omega than with
@@ -100,27 +108,20 @@ static inline string make_parentterm(const string& udi)
 bool Db::Native::subDocs(const string &udi, vector<Xapian::docid>& docids) 
 {
     LOGDEB2(("subDocs: [%s]\n", uniterm.c_str()));
-    string ermsg;
     string pterm = make_parentterm(udi);
-    for (int tries = 0; tries < 2; tries++) {
-	try {
-	    Xapian::PostingIterator it = db.postlist_begin(pterm);
-	    for (; it != db.postlist_end(pterm); it++) {
-		docids.push_back(*it);
-	    }
-	    LOGDEB(("Db::Native::subDocs: returning %d ids\n", docids.size()));
-	    return true;
-	} catch (const Xapian::DatabaseModifiedError &e) {
-	    LOGDEB(("Db::subDocs: got modified error. reopen/retry\n"));
-	    // Can't use reOpen() here, I'm a Native:: method, this
-	    // would delete my own object
-	    db = Xapian::Database(m_db->m_basedir);
-	} XCATCHERROR(ermsg);
-	if (!ermsg.empty()) 
-	    break;
+
+    XAPTRY(docids.clear();
+           docids.insert(docids.begin(), xrdb.postlist_begin(pterm), 
+                         xrdb.postlist_end(pterm)),
+           xrdb, m_rcldb->m_reason);
+
+    if (!m_rcldb->m_reason.empty()) {
+        LOGERR(("Rcl::Db::subDocs: %s\n", m_rcldb->m_reason.c_str()));
+        return false;
+    } else {
+        LOGDEB0(("Db::Native::subDocs: returning %d ids\n", docids.size()));
+        return true;
     }
-    LOGERR(("Rcl::Db::subDocs: %s\n", ermsg.c_str()));
-    return false;
 }
 
 // Only ONE field name inside the index data record differs from the
@@ -133,7 +134,7 @@ bool Db::Native::dbDataToRclDoc(Xapian::docid docid, std::string &data,
 				Doc &doc, int percent)
 {
     LOGDEB0(("Db::dbDataToRclDoc: data: %s\n", data.c_str()));
-    ConfSimple parms(&data);
+    ConfSimple parms(data);
     if (!parms.ok())
 	return false;
     parms.get(Doc::keyurl, doc.url);
@@ -171,9 +172,9 @@ bool Db::Native::dbDataToRclDoc(Xapian::docid docid, std::string &data,
     return true;
 }
 
-static list<string> noPrefixList(const list<string>& in) 
+// Remove prefixes (caps) from a list of terms.
+static void noPrefixList(const list<string>& in, list<string>& out) 
 {
-    list<string> out;
     for (list<string>::const_iterator qit = in.begin(); 
 	 qit != in.end(); qit++) {
 	if ('A' <= qit->at(0) && qit->at(0) <= 'Z') {
@@ -187,40 +188,62 @@ static list<string> noPrefixList(const list<string>& in)
 	    out.push_back(*qit);
 	}
     }
-    return out;
 }
 
-    //#define DEBUGABSTRACT  1
+//#define DEBUGABSTRACT  1
 #ifdef DEBUGABSTRACT
 #define LOGABS LOGDEB
 #else
 #define LOGABS LOGDEB2
 #endif
+#if 0
+static void listList(const string& what, const list<string>&l)
+{
+    string a;
+    for (list<string>::const_iterator it = l.begin(); it != l.end(); it++) {
+        a = a + *it + " ";
+    }
+    LOGDEB(("%s: %s\n", what.c_str(), a.c_str()));
+}
+#endif
 
 // Build a document abstract by extracting text chunks around the query terms
 // This uses the db termlists, not the original document.
+//
+// DatabaseModified and other general exceptions are catched and
+// possibly retried by our caller
 string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
 {
     Chrono chron;
     LOGDEB(("makeAbstract:%d: maxlen %d wWidth %d\n", chron.ms(),
-	     m_db->m_synthAbsLen, m_db->m_synthAbsWordCtxLen));
+	     m_rcldb->m_synthAbsLen, m_rcldb->m_synthAbsWordCtxLen));
 
-    list<string> iterms;
-    query->getQueryTerms(iterms);
+    list<string> terms;
 
-    list<string> terms = noPrefixList(iterms);
-    if (terms.empty()) {
-	return string();
+    {
+        list<string> iterms;
+        query->getMatchTerms(docid, iterms);
+        noPrefixList(iterms, terms);
+        if (terms.empty()) {
+            LOGDEB(("makeAbstract::Empty term list\n"));
+            return string();
+        }
     }
+//    listList("Match terms: ", terms);
 
-    // Retrieve db-wide frequencies for the query terms
+    // Retrieve db-wide frequencies for the query terms (we do this once per
+    // query, using all the query terms, not only the document match terms)
     if (query->m_nq->termfreqs.empty()) {
-	double doccnt = db.get_doccount();
+        list<string> iqterms, qterms;
+        query->getQueryTerms(iqterms);
+        noPrefixList(iqterms, qterms);
+//        listList("Query terms: ", qterms);
+	double doccnt = xrdb.get_doccount();
 	if (doccnt == 0) doccnt = 1;
-	for (list<string>::const_iterator qit = terms.begin(); 
-	     qit != terms.end(); qit++) {
-	    query->m_nq->termfreqs[*qit] = db.get_termfreq(*qit) / doccnt;
-	    LOGABS(("makeAbstract: [%s] db freq %.1e\n", qit->c_str(), 
+	for (list<string>::const_iterator qit = qterms.begin(); 
+	     qit != qterms.end(); qit++) {
+	    query->m_nq->termfreqs[*qit] = xrdb.get_termfreq(*qit) / doccnt;
+	    LOGDEB(("makeAbstract: [%s] db freq %.1e\n", qit->c_str(), 
 		    query->m_nq->termfreqs[*qit]));
 	}
 	LOGABS(("makeAbstract:%d: got termfreqs\n", chron.ms()));
@@ -232,13 +255,13 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
     // and show text around the less common search terms.
     map<string, double> termQcoefs;
     double totalweight = 0;
-    double doclen = db.get_doclength(docid);
+    double doclen = xrdb.get_doclength(docid);
     if (doclen == 0) doclen = 1;
     for (list<string>::const_iterator qit = terms.begin(); 
 	 qit != terms.end(); qit++) {
-	Xapian::TermIterator term = db.termlist_begin(docid);
+	Xapian::TermIterator term = xrdb.termlist_begin(docid);
 	term.skip_to(*qit);
-	if (term != db.termlist_end(docid) && *term == *qit) {
+	if (term != xrdb.termlist_end(docid) && *term == *qit) {
 	    double q = (term.get_wdf() / doclen) * query->m_nq->termfreqs[*qit];
 	    q = -log10(q);
 	    if (q < 3) {
@@ -292,7 +315,7 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
     // with words. We used to limit the character size at the end, but
     // this damaged our careful selection of terms
     const unsigned int maxtotaloccs = 
-	m_db->m_synthAbsLen /(7 * (m_db->m_synthAbsWordCtxLen+1));
+	m_rcldb->m_synthAbsLen /(7 * (m_rcldb->m_synthAbsWordCtxLen+1));
     LOGABS(("makeAbstract:%d: mxttloccs %d\n", chron.ms(), maxtotaloccs));
     // This can't happen, but would crash us
     if (totalweight == 0.0) {
@@ -328,8 +351,8 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
 	string emptys;
 	try {
 	    unsigned int occurrences = 0;
-	    for (pos = db.positionlist_begin(docid, qterm); 
-		 pos != db.positionlist_end(docid, qterm); pos++) {
+	    for (pos = xrdb.positionlist_begin(docid, qterm); 
+		 pos != xrdb.positionlist_end(docid, qterm); pos++) {
 		int ipos = *pos;
 		if (ipos < int(baseTextPosition)) // Not in text body
 		    continue;
@@ -342,9 +365,9 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
 		// step by inserting empty strings. Special provisions
 		// for adding ellipsis and for positions overlapped by
 		// the match term.
-		unsigned int sta = MAX(0, ipos-m_db->m_synthAbsWordCtxLen);
+		unsigned int sta = MAX(0, ipos-m_rcldb->m_synthAbsWordCtxLen);
 		unsigned int sto = ipos + qtrmwrdcnt-1 + 
-		    m_db->m_synthAbsWordCtxLen;
+		    m_rcldb->m_synthAbsWordCtxLen;
 		for (unsigned int ii = sta; ii <= sto;  ii++) {
 		    if (ii == (unsigned int)ipos) {
 			sparseDoc[ii] = qterm;
@@ -394,8 +417,8 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
 	Xapian::TermIterator term;
 	int cutoff = 500 * 1000;
 
-	for (term = db.termlist_begin(docid);
-	     term != db.termlist_end(docid); term++) {
+	for (term = xrdb.termlist_begin(docid);
+	     term != xrdb.termlist_end(docid); term++) {
 	    // Ignore prefixed terms
 	    if ('A' <= (*term).at(0) && (*term).at(0) <= 'Z')
 		continue;
@@ -405,8 +428,8 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
 	    }
 
 	    Xapian::PositionIterator pos;
-	    for (pos = db.positionlist_begin(docid, *term); 
-		 pos != db.positionlist_end(docid, *term); pos++) {
+	    for (pos = xrdb.positionlist_begin(docid, *term); 
+		 pos != xrdb.positionlist_end(docid, *term); pos++) {
 		if (cutoff-- < 0) {
 		    LOGDEB(("makeAbstract: max term count cutoff\n"));
 		    break;
@@ -445,7 +468,7 @@ string Db::Native::makeAbstract(Xapian::docid docid, Query *query)
     }
 #endif
 
-    LOGDEB(("makeAbstract:%d: extracting\n", chron.millis()));
+    LOGABS(("makeAbstract:%d: extracting\n", chron.millis()));
 
     // Finally build the abstract by walking the map (in order of position)
     string abstract;
@@ -495,7 +518,7 @@ Db::~Db()
     LOGDEB2(("Db::~Db\n"));
     if (m_ndb == 0)
 	return;
-    LOGDEB(("Db::~Db: isopen %d m_iswritable %d\n", m_ndb->m_isopen, 
+    LOGDEB(("Db::~Db: isopen %d m_iswritable %d\n", m_ndb->m_isopen,
 	    m_ndb->m_iswritable));
     i_close(true);
 }
@@ -507,7 +530,7 @@ list<string> Db::getStemmerNames()
     return res;
 }
 
-bool Db::open(OpenMode mode, bool keep_updated)
+bool Db::open(OpenMode mode)
 {
     if (m_ndb == 0 || m_config == 0) {
 	m_reason = "Null configuration or Xapian Db";
@@ -532,28 +555,26 @@ bool Db::open(OpenMode mode, bool keep_updated)
 	    {
 		int action = (mode == DbUpd) ? Xapian::DB_CREATE_OR_OPEN :
 		    Xapian::DB_CREATE_OR_OVERWRITE;
-		m_ndb->wdb = Xapian::WritableDatabase(dir, action);
+		m_ndb->xwdb = Xapian::WritableDatabase(dir, action);
 		m_ndb->m_iswritable = true;
 		// We open a readonly object in all cases (possibly in
 		// addition to the r/w one) because some operations
 		// are faster when performed through a Database: no
 		// forced flushes on allterms_begin(), ie, used in
 		// subDocs()
-		m_ndb->db = Xapian::Database(dir);
+		m_ndb->xrdb = Xapian::Database(dir);
 		LOGDEB(("Db::open: lastdocid: %d\n", 
-			m_ndb->wdb.get_lastdocid()));
-		if (!keep_updated) {
-		    LOGDEB2(("Db::open: resetting updated\n"));
-		    updated.resize(m_ndb->wdb.get_lastdocid() + 1);
-		    for (unsigned int i = 0; i < updated.size(); i++)
-			updated[i] = false;
-		}
+			m_ndb->xwdb.get_lastdocid()));
+                LOGDEB2(("Db::open: resetting updated\n"));
+                updated.resize(m_ndb->xwdb.get_lastdocid() + 1);
+                for (unsigned int i = 0; i < updated.size(); i++)
+                    updated[i] = false;
 	    }
 	    break;
 	case DbRO:
 	default:
 	    m_ndb->m_iswritable = false;
-	    m_ndb->db = Xapian::Database(dir);
+	    m_ndb->xrdb = Xapian::Database(dir);
 	    for (list<string>::iterator it = m_extraDbs.begin();
 		 it != m_extraDbs.end(); it++) {
 		string aerr;
@@ -561,7 +582,7 @@ bool Db::open(OpenMode mode, bool keep_updated)
 		aerr.erase();
 		try {
 		    // Make this non-fatal
-		    m_ndb->db.add_database(Xapian::Database(*it));
+		    m_ndb->xrdb.add_database(Xapian::Database(*it));
 		} XCATCHERROR(aerr);
 		if (!aerr.empty())
 		    LOGERR(("Db::Open: error while trying to add database "
@@ -571,9 +592,8 @@ bool Db::open(OpenMode mode, bool keep_updated)
 	}
 	// Check index format version. Must not try to check a just created or
 	// truncated db
-	if (mode != DbTrunc && m_ndb->db.get_doccount()>0) {
-	    Xapian::Database cdb = m_ndb->m_iswritable ? m_ndb->wdb: m_ndb->db;
-	    string version = cdb.get_metadata(RCL_IDX_VERSION_KEY);
+	if (mode != DbTrunc && m_ndb->xdb().get_doccount() > 0) {
+	    string version = m_ndb->xdb().get_metadata(RCL_IDX_VERSION_KEY);
 	    if (version.compare(RCL_IDX_VERSION)) {
 		m_ndb->m_noversionwrite = true;
 		LOGERR(("Rcl::Db::open: file index [%s], software [%s]\n",
@@ -598,7 +618,6 @@ bool Db::close()
     LOGDEB2(("Db::close()\n"));
     return i_close(false);
 }
-
 bool Db::i_close(bool final)
 {
     if (m_ndb == 0)
@@ -613,7 +632,7 @@ bool Db::i_close(bool final)
 	bool w = m_ndb->m_iswritable;
 	if (w) {
 	    if (!m_ndb->m_noversionwrite)
-		m_ndb->wdb.set_metadata(RCL_IDX_VERSION_KEY, RCL_IDX_VERSION);
+		m_ndb->xwdb.set_metadata(RCL_IDX_VERSION_KEY, RCL_IDX_VERSION);
 	    LOGDEB(("Rcl::Db:close: xapian will close. May take some time\n"));
 	}
 	// Used to do a flush here. Cant see why it should be necessary.
@@ -634,12 +653,17 @@ bool Db::i_close(bool final)
     return false;
 }
 
-bool Db::reOpen()
+// Reopen the db with a changed list of additional dbs
+bool Db::adjustdbs()
 {
+    if (m_mode != DbRO) {
+        LOGERR(("Db::adjustdbs: mode not RO\n"));
+        return false;
+    }
     if (m_ndb && m_ndb->m_isopen) {
 	if (!close())
 	    return false;
-	if (!open(m_mode, true)) {
+	if (!open(m_mode)) {
 	    return false;
 	}
     }
@@ -649,19 +673,14 @@ bool Db::reOpen()
 int Db::docCnt()
 {
     int res = -1;
-    string ermsg;
-    if (m_ndb && m_ndb->m_isopen) {
-	try {
-	    res = m_ndb->m_iswritable ? m_ndb->wdb.get_doccount() : 
-		m_ndb->db.get_doccount();
-	} catch (const Xapian::DatabaseModifiedError &e) {
-	    LOGDEB(("Db::docCnt: got modified error. reopen/retry\n"));
-	    reOpen();
-	    res = m_ndb->m_iswritable ? m_ndb->wdb.get_doccount() : 
-		m_ndb->db.get_doccount();
-	} XCATCHERROR(ermsg);
-	if (!ermsg.empty())
-	    LOGERR(("Db::docCnt: got error: %s\n", ermsg.c_str()));
+    if (!m_ndb || !m_ndb->m_isopen)
+        return -1;
+
+    XAPTRY(res = m_ndb->xdb().get_doccount(), m_ndb->xrdb, m_reason);
+
+    if (!m_reason.empty()) {
+        LOGERR(("Db::docCnt: got error: %s\n", m_reason.c_str()));
+        return -1;
     }
     return res;
 }
@@ -677,7 +696,7 @@ bool Db::addQueryDb(const string &dir)
     if (find(m_extraDbs.begin(), m_extraDbs.end(), dir) == m_extraDbs.end()) {
 	m_extraDbs.push_back(dir);
     }
-    return reOpen();
+    return adjustdbs();
 }
 
 bool Db::rmQueryDb(const string &dir)
@@ -695,7 +714,7 @@ bool Db::rmQueryDb(const string &dir)
 	    m_extraDbs.erase(it);
 	}
     }
-    return reOpen();
+    return adjustdbs();
 }
 
 bool Db::testDbDir(const string &dir)
@@ -878,7 +897,8 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 
     // Split and index file name as document term(s)
     LOGDEB2(("Db::add: split file name [%s]\n", fn.c_str()));
-    splitter.text_to_words(doc.utf8fn);
+    if (!splitter.text_to_words(doc.utf8fn))
+        LOGDEB(("Db::addOrUpdate: split failed for file name\n"));
     splitData.basepos += splitData.curpos + 100;
 
     // Index textual metadata.  These are all indexed as text with
@@ -900,7 +920,9 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 		    meta_it->first.c_str(), pfx.c_str(), 
 		    meta_it->second.c_str()));
 	    splitData.setprefix(pfx); // Subject
-	    splitter.text_to_words(meta_it->second);
+	    if (!splitter.text_to_words(meta_it->second))
+                LOGDEB(("Db::addOrUpdate: split failed for %s\n", 
+                        meta_it->first.c_str()));
 	    splitData.setprefix(string());
 	    splitData.basepos += splitData.curpos + 100;
 	}
@@ -913,7 +935,8 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 
     // Split and index body text
     LOGDEB2(("Db::add: split body\n"));
-    splitter.text_to_words(doc.text);
+    if (!splitter.text_to_words(doc.text))
+        LOGDEB(("Db::addOrUpdate: split failed for main text\n"));
 
     ////// Special terms for other metadata. No positions for these.
     // Mime type
@@ -1061,7 +1084,7 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
     // Add db entry or update existing entry:
     try {
 	Xapian::docid did = 
-	    m_ndb->wdb.replace_document(uniterm, newdocument);
+	    m_ndb->xwdb.replace_document(uniterm, newdocument);
 	if (did < updated.size()) {
 	    updated[did] = true;
 	    LOGDEB(("Db::add: docid %d updated [%s]\n", did, fnc));
@@ -1075,7 +1098,7 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 	ermsg.erase();
 	// FIXME: is this ever actually needed?
 	try {
-	    m_ndb->wdb.add_document(newdocument);
+	    m_ndb->xwdb.add_document(newdocument);
 	    LOGDEB(("Db::add: %s added (failed re-seek for duplicate)\n", 
 		    fnc));
 	} XCATCHERROR(ermsg);
@@ -1092,7 +1115,7 @@ bool Db::addOrUpdate(const string &udi, const string &parent_udi,
 	    ermsg.erase();
 	    LOGDEB(("Db::add: text size >= %d Mb, flushing\n", m_flushMb));
 	    try {
-		m_ndb->wdb.flush();
+		m_ndb->xwdb.flush();
 	    } XCATCHERROR(ermsg);
 	    if (!ermsg.empty()) {
 		LOGERR(("Db::add: flush() failed: %s\n", ermsg.c_str()));
@@ -1122,13 +1145,13 @@ bool Db::needUpdate(const string &udi, const string& sig)
     for (int tries = 0; tries < 2; tries++) {
 	try {
 	    // Get the doc or pseudo-doc
-	    Xapian::PostingIterator docid = m_ndb->db.postlist_begin(uniterm);
-	    if (docid == m_ndb->db.postlist_end(uniterm)) {
+	    Xapian::PostingIterator docid =m_ndb->xrdb.postlist_begin(uniterm);
+	    if (docid == m_ndb->xrdb.postlist_end(uniterm)) {
 		// If no document exist with this path, we do need update
-		LOGDEB(("Db::needUpdate: no path: [%s]\n", uniterm.c_str()));
+		LOGDEB(("Db::needUpdate:yes (new): [%s]\n", uniterm.c_str()));
 		return true;
 	    }
-	    Xapian::Document doc = m_ndb->db.get_document(*docid);
+	    Xapian::Document doc = m_ndb->xrdb.get_document(*docid);
 
 	    // Retrieve old file/doc signature from value
 	    string osig = doc.get_value(VALUE_SIG);
@@ -1140,9 +1163,9 @@ bool Db::needUpdate(const string &udi, const string& sig)
 			osig.c_str(), sig.c_str()));
 		// Db is not up to date. Let's index the file
 		return true;
-	    } 
+	    }
 
-	    LOGDEB(("Db::needUpdate: uptodate: [%s]\n", uniterm.c_str()));
+	    LOGDEB(("Db::needUpdate:no: [%s]\n", uniterm.c_str()));
 
 	    // Up to date. 
 
@@ -1165,13 +1188,14 @@ bool Db::needUpdate(const string &udi, const string& sig)
 	    return false;
 	} catch (const Xapian::DatabaseModifiedError &e) {
 	    LOGDEB(("Db::needUpdate: got modified error. reopen/retry\n"));
-	    reOpen();
-	} XCATCHERROR(ermsg);
-	if (!ermsg.empty())
-	    break;
+            m_reason = e.get_msg();
+	    m_ndb->xrdb.reopen();
+            continue;
+	} XCATCHERROR(m_reason);
+        break;
     }
     LOGERR(("Db::needUpdate: error while checking existence: %s\n", 
-	    ermsg.c_str()));
+	    m_reason.c_str()));
     return true;
 }
 
@@ -1210,8 +1234,7 @@ bool Db::createStemDb(const string& lang)
     if (m_ndb == 0 || m_ndb->m_isopen == false)
 	return false;
 
-    return StemDb::createDb(m_ndb->m_iswritable ? m_ndb->wdb : m_ndb->db, 
-			     m_basedir, lang);
+    return StemDb::createDb(m_ndb->xdb(), m_basedir, lang);
 }
 
 /**
@@ -1237,7 +1260,7 @@ bool Db::purge()
     // that any added document would go to the index. Kept here
     // because it doesn't really hurt.
     try {
-	m_ndb->wdb.flush();
+	m_ndb->xwdb.flush();
     } catch (...) {
 	LOGERR(("Db::purge: 1st flush failed\n"));
 
@@ -1248,7 +1271,7 @@ bool Db::purge()
     for (Xapian::docid docid = 1; docid < updated.size(); ++docid) {
 	if (!updated[docid]) {
 	    try {
-		m_ndb->wdb.delete_document(docid);
+		m_ndb->xwdb.delete_document(docid);
 		LOGDEB(("Db::purge: deleted document #%d\n", docid));
 	    } catch (const Xapian::DocNotFoundError &) {
 		LOGDEB0(("Db::purge: document #%d not found\n", docid));
@@ -1261,7 +1284,7 @@ bool Db::purge()
     }
 
     try {
-	m_ndb->wdb.flush();
+	m_ndb->xwdb.flush();
     } catch (...) {
 	LOGERR(("Db::purge: 2nd flush failed\n"));
     }
@@ -1269,18 +1292,22 @@ bool Db::purge()
 }
 
 /* Delete document(s) for given unique identifier (doc and descendents) */
-bool Db::purgeFile(const string &udi)
+bool Db::purgeFile(const string &udi, bool *existed)
 {
     LOGDEB(("Db:purgeFile: [%s]\n", udi.c_str()));
-    if (m_ndb == 0)
+    if (m_ndb == 0 || !m_ndb->m_iswritable)
 	return false;
-    Xapian::WritableDatabase db = m_ndb->wdb;
+    Xapian::WritableDatabase db = m_ndb->xwdb;
     string uniterm = make_uniterm(udi);
     string ermsg;
     try {
 	Xapian::PostingIterator docid = db.postlist_begin(uniterm);
-	if (docid == db.postlist_end(uniterm))
+	if (docid == db.postlist_end(uniterm)) {
+            if (existed)
+                *existed = false;
 	    return true;
+        }
+        *existed = true;
 	LOGDEB(("purgeFile: delete docid %d\n", *docid));
 	db.delete_document(*docid);
 	vector<Xapian::docid> docids;
@@ -1302,11 +1329,7 @@ bool Db::purgeFile(const string &udi)
 // File name wild card expansion. This is a specialisation ot termMatch
 bool Db::filenameWildExp(const string& fnexp, list<string>& names)
 {
-    string pattern;
-    if (!unacmaybefold(fnexp, pattern, "UTF-8", true)) {
-	LOGERR(("Db::filenameWildExp: unac error for [%s]\n", fnexp.c_str()));
-	return false;
-    }
+    string pattern = fnexp;
     names.clear();
 
     // If pattern is not quoted, and has no wildcards, we add * at
@@ -1323,12 +1346,12 @@ bool Db::filenameWildExp(const string& fnexp, list<string>& names)
 	return false;
     for (list<TermMatchEntry>::const_iterator it = entries.begin();
 	 it != entries.end(); it++) 
-	names.push_back("XSFN"+it->term);
+	names.push_back(it->term);
 
     if (names.empty()) {
 	// Build an impossible query: we know its impossible because we
 	// control the prefixes!
-	names.push_back("XIMPOSSIBLE");
+	names.push_back("XNONENoMatchingTerms");
     }
     return true;
 }
@@ -1371,6 +1394,16 @@ bool Db::stemExpand(const string &lang, const string &term,
     return true;
 }
 
+/** Add prefix to all strings in list */
+static void addPrefix(list<TermMatchEntry>& terms, const string& prefix)
+{
+    if (prefix.empty())
+	return;
+    for (list<TermMatchEntry>::iterator it = terms.begin(); 
+         it != terms.end(); it++)
+	it->term.insert(0, prefix);
+}
+
 // Characters that can begin a wildcard or regexp expression. We use skipto
 // to begin the allterms search with terms that begin with the portion of
 // the input string prior to these chars.
@@ -1382,11 +1415,13 @@ bool Db::termMatch(MatchType typ, const string &lang,
 		   const string &root, 
 		   list<TermMatchEntry>& res,
 		   int max, 
-		   const string& field)
+		   const string& field,
+                   string *prefixp
+    )
 {
     if (!m_ndb || !m_ndb->m_isopen)
 	return false;
-    Xapian::Database db = m_ndb->m_iswritable ? m_ndb->wdb: m_ndb->db;
+    Xapian::Database xdb = m_ndb->xdb();
 
     res.clear();
 
@@ -1401,6 +1436,12 @@ bool Db::termMatch(MatchType typ, const string &lang,
     string prefix;
     if (!field.empty()) {
 	(void)fieldToPrefix(field, prefix); 
+        if (prefix.empty()) {
+            LOGDEB(("Db::termMatch: field is not indexed (no prefix): [%s]\n", 
+                    field.c_str()));
+        }
+        if (prefixp)
+            *prefixp = prefix;
     }
 
     if (typ == ET_STEM) {
@@ -1410,9 +1451,14 @@ bool Db::termMatch(MatchType typ, const string &lang,
 	res.unique();
 	for (list<TermMatchEntry>::iterator it = res.begin(); 
 	     it != res.end(); it++) {
-	    it->wcf = db.get_collection_freq(it->term);
+	    XAPTRY(it->wcf = xdb.get_collection_freq(it->term),
+                   xdb, m_reason);
+            if (!m_reason.empty())
+                return false;
 	    LOGDEB1(("termMatch: %d [%s]\n", it->wcf, it->term.c_str()));
 	}
+        if (!prefix.empty())
+            addPrefix(res, prefix);
     } else {
 	regex_t reg;
 	int errcode;
@@ -1439,34 +1485,44 @@ bool Db::termMatch(MatchType typ, const string &lang,
 	}
 	LOGDEB(("termMatch: initsec: [%s]\n", is.c_str()));
 
-	string ermsg;
-	try {
-	    Xapian::TermIterator it = db.allterms_begin(); 
-	    if (!is.empty())
-		it.skip_to(is.c_str());
-	    for (int n = 0; it != db.allterms_end(); it++) {
-		// If we're beyond the terms matching the initial string, end
-		if (!is.empty() && (*it).find(is) != 0)
-		    break;
-		string term;
-		if (!prefix.empty())
-		    term = (*it).substr(prefix.length());
-		else
-		    term = *it;
-		if (typ == ET_WILD) {
-		    if (fnmatch(droot.c_str(), term.c_str(), 0) == FNM_NOMATCH)
-			continue;
-		} else {
-		    if (regexec(&reg, term.c_str(), 0, 0, 0))
-			continue;
-		}
-		// Do we want stem expansion here? We don't do it for now
-		res.push_back(TermMatchEntry(term, it.get_termfreq()));
-		++n;
-	    }
-	} XCATCHERROR(ermsg);
-	if (!ermsg.empty()) {
-	    LOGERR(("termMatch: %s\n", ermsg.c_str()));
+        for (int tries = 0; tries < 2; tries++) { 
+            try {
+                Xapian::TermIterator it = xdb.allterms_begin(); 
+                if (!is.empty())
+                    it.skip_to(is.c_str());
+                for (int n = 0; it != xdb.allterms_end(); it++) {
+                    // If we're beyond the terms matching the initial
+                    // string, end
+                    if (!is.empty() && (*it).find(is) != 0)
+                        break;
+                    string term;
+                    if (!prefix.empty())
+                        term = (*it).substr(prefix.length());
+                    else
+                        term = *it;
+                    if (typ == ET_WILD) {
+                        if (fnmatch(droot.c_str(), term.c_str(), 0) == 
+                            FNM_NOMATCH)
+                            continue;
+                    } else {
+                        if (regexec(&reg, term.c_str(), 0, 0, 0))
+                            continue;
+                    }
+                    // Do we want stem expansion here? We don't do it for now
+                    res.push_back(TermMatchEntry(*it, it.get_termfreq()));
+                    ++n;
+                }
+                m_reason.erase();
+                break;
+            } catch (const Xapian::DatabaseModifiedError &e) {
+                m_reason = e.get_msg();
+                xdb.reopen();
+                continue;
+            } XCATCHERROR(m_reason);
+            break;
+        }
+	if (!m_reason.empty()) {
+	    LOGERR(("termMatch: %s\n", m_reason.c_str()));
 	    return false;
 	}
 
@@ -1500,13 +1556,10 @@ TermIter *Db::termWalkOpen()
 	return 0;
     TermIter *tit = new TermIter;
     if (tit) {
-	tit->db = m_ndb->m_iswritable ? m_ndb->wdb: m_ndb->db;
-	string ermsg;
-	try {
-	    tit->it = tit->db.allterms_begin();
-	} XCATCHERROR(ermsg);
-	if (!ermsg.empty()) {
-	    LOGERR(("Db::termWalkOpen: xapian error: %s\n", ermsg.c_str()));
+	tit->db = m_ndb->xdb();
+        XAPTRY(tit->it = tit->db.allterms_begin(), tit->db, m_reason);
+	if (!m_reason.empty()) {
+	    LOGERR(("Db::termWalkOpen: xapian error: %s\n", m_reason.c_str()));
 	    return 0;
 	}
     }
@@ -1514,15 +1567,15 @@ TermIter *Db::termWalkOpen()
 }
 bool Db::termWalkNext(TermIter *tit, string &term)
 {
-    string ermsg;
-    try {
+    XAPTRY(
 	if (tit && tit->it != tit->db.allterms_end()) {
 	    term = *(tit->it)++;
 	    return true;
 	}
-    } XCATCHERROR(ermsg);
-    if (!ermsg.empty()) {
-	LOGERR(("Db::termWalkOpen: xapian error: %s\n", ermsg.c_str()));
+        , tit->db, m_reason);
+
+    if (!m_reason.empty()) {
+	LOGERR(("Db::termWalkOpen: xapian error: %s\n", m_reason.c_str()));
     }
     return false;
 }
@@ -1537,14 +1590,12 @@ bool Db::termExists(const string& word)
 {
     if (!m_ndb || !m_ndb->m_isopen)
 	return 0;
-    Xapian::Database db = m_ndb->m_iswritable ? m_ndb->wdb: m_ndb->db;
-    string ermsg;
-    try {
-	if (!db.term_exists(word))
-	    return false;
-    } XCATCHERROR(ermsg);
-    if (!ermsg.empty()) {
-	LOGERR(("Db::termWalkOpen: xapian error: %s\n", ermsg.c_str()));
+
+    XAPTRY(if (!m_ndb->xdb().term_exists(word)) return false,
+           m_ndb->xrdb, m_reason);
+
+    if (!m_reason.empty()) {
+	LOGERR(("Db::termWalkOpen: xapian error: %s\n", m_reason.c_str()));
 	return false;
     }
     return true;
@@ -1567,15 +1618,19 @@ bool Db::stemDiffers(const string& lang, const string& word,
 bool Db::makeDocAbstract(Doc &doc, Query *query, string& abstract)
 {
     LOGDEB1(("Db::makeDocAbstract: exti %d\n", exti));
-    if (!m_ndb) {
+    if (!m_ndb || !m_ndb->m_isopen) {
 	LOGERR(("Db::makeDocAbstract: no db\n"));
 	return false;
     }
-    abstract = m_ndb->makeAbstract(doc.xdocid, query);
-    return true;
+
+    XAPTRY(abstract = m_ndb->makeAbstract(doc.xdocid, query),
+           m_ndb->xrdb, m_reason);
+
+    return m_reason.empty() ? true : false;
 }
 
-// Retrieve document defined by file name and internal path. 
+// Retrieve document defined by Unique doc identifier. This is mainly used
+// by the GUI history feature
 bool Db::getDoc(const string &udi, Doc &doc)
 {
     LOGDEB(("Db:getDoc: [%s]\n", udi.c_str()));
@@ -1587,25 +1642,33 @@ bool Db::getDoc(const string &udi, Doc &doc)
     doc.pc = 100;
 
     string uniterm = make_uniterm(udi);
-    string ermsg;
-    try {
-	if (!m_ndb->db.term_exists(uniterm)) {
-	    // Document found in history no longer in the database.
-	    // We return true (because their might be other ok docs further)
-	    // but indicate the error with pc = -1
-	    doc.pc = -1;
-	    LOGINFO(("Db:getDoc: no such doc in index: [%s] (len %d)\n",
-		     uniterm.c_str(), uniterm.length()));
-	    return true;
-	}
-	Xapian::PostingIterator docid = m_ndb->db.postlist_begin(uniterm);
-	Xapian::Document xdoc = m_ndb->db.get_document(*docid);
-	string data = xdoc.get_data();
-	return m_ndb->dbDataToRclDoc(*docid, data, doc, 100);
-    } XCATCHERROR(ermsg);
-    if (!ermsg.empty()) {
-	LOGERR(("Db::getDoc: %s\n", ermsg.c_str()));
+    for (int tries = 0; tries < 2; tries++) {
+	try {
+            if (!m_ndb->xrdb.term_exists(uniterm)) {
+                // Document found in history no longer in the
+                // database.  We return true (because their might be
+                // other ok docs further) but indicate the error with
+                // pc = -1
+                doc.pc = -1;
+                LOGINFO(("Db:getDoc: no such doc in index: [%s] (len %d)\n",
+                         uniterm.c_str(), uniterm.length()));
+                return true;
+            }
+            Xapian::PostingIterator docid = 
+                m_ndb->xrdb.postlist_begin(uniterm);
+            Xapian::Document xdoc = m_ndb->xrdb.get_document(*docid);
+            string data = xdoc.get_data();
+            doc.meta[Rcl::Doc::keyudi] = udi;
+            return m_ndb->dbDataToRclDoc(*docid, data, doc, 100);
+	} catch (const Xapian::DatabaseModifiedError &e) {
+            m_reason = e.get_msg();
+	    m_ndb->xrdb.reopen();
+            continue;
+	} XCATCHERROR(m_reason);
+        break;
     }
+
+    LOGERR(("Db::getDoc: %s\n", m_reason.c_str()));
     return false;
 }
 
