@@ -44,7 +44,7 @@ using std::vector;
 // user interface. In other words, this is not exhaustive or well-designed or 
 // reusable.
 //
-// Unique Document Identifier: unically identifies a document in its
+// Unique Document Identifier: uniquely identifies a document in its
 // source storage (file system or other). Used for up to date checks
 // etc. "udi". Our user is responsible for making sure it's not too
 // big, cause it's stored as a Xapian term (< 150 bytes would be
@@ -86,7 +86,7 @@ class Db {
     ~Db();
 
     enum OpenMode {DbRO, DbUpd, DbTrunc};
-    bool open(OpenMode mode, bool keep_updated = false);
+    bool open(OpenMode mode);
     bool close();
     bool isopen();
 
@@ -121,7 +121,7 @@ class Db {
 		     const Doc &doc);
 
     /** Delete document(s) for given UDI, including subdocs */
-    bool purgeFile(const string &udi);
+    bool purgeFile(const string &udi, bool *existed = 0);
 
     /** Remove documents that no longer exist in the file system. This
      * depends on the update map, which is built during
@@ -158,7 +158,9 @@ class Db {
     enum MatchType {ET_WILD, ET_REGEXP, ET_STEM};
     bool termMatch(MatchType typ, const string &lang, const string &s, 
 		   list<TermMatchEntry>& result, int max = -1, 
-		   const string& field = "");
+		   const string& field = "",
+                   string *prefix = 0
+        );
 
     /** Special filename wildcard to XSFN terms expansion.
 	internal/searchdata use only */
@@ -233,8 +235,9 @@ private:
     vector<bool> updated;
 
     StopList m_stops;
-
-    bool reOpen(); // Close/open, same mode/opts
+    
+    // Reinitialize when adding/removing additional dbs
+    bool adjustdbs(); 
     bool stemExpand(const string &lang, const string &s, 
 		    list<TermMatchEntry>& result, int max = -1);
 
@@ -243,6 +246,10 @@ private:
     Db& operator=(const Db &) {return *this;};
 };
 
+// This has to go somewhere, and as it needs the Xapian version, this is
+// the most reasonable place.
+string version_string();
+
 #ifndef NO_NAMESPACES
 }
 #endif // NO_NAMESPACES
