@@ -40,7 +40,7 @@ static const char Pad64 = '=';
 bool base64_decode(const string& in, string& out)
 {
     int io = 0, state = 0, ch = 0;
-    char *pos;
+    const char *pos;
     unsigned int ii = 0;
     out.erase();
     out.reserve(in.length());
@@ -125,7 +125,7 @@ bool base64_decode(const string& in, string& out)
 	    for (; ii < in.length(); ch = in[ii++])
 		if (!isspace((unsigned char)ch)) {
 		    DPRINT((stderr, "base64_dec: non-white at eod: 0x%x\n", 
-			    (unsigned int)ch));
+			    (unsigned int)((unsigned char)ch)));
 		    // Well, there are bad encoders out there. Let it pass
 		    //return false;
 		}
