@@ -25,6 +25,7 @@ static char rcsid [] = "@(#$Id: conftree.cpp,v 1.16 2008-07-01 11:51:51 dockes E
 
 #include <unistd.h> // for access(2)
 #include <ctype.h>
+#include <fnmatch.h>
 
 #include <fstream>
 #include <sstream>
@@ -45,7 +46,7 @@ using std::list;
 #define MIN(A,B) ((A)<(B) ? (A) : (B))
 #endif
 
-//#define DEBUG
+#undef DEBUG
 #ifdef DEBUG
 #define LOGDEB(X) fprintf X
 #else
@@ -59,24 +60,32 @@ void ConfSimple::parseinput(istream &input)
     char cline[LL];
     bool appending = false;
     string line;
+    bool eof = false;
 
     for (;;) {
+        cline[0] = 0;
 	input.getline(cline, LL-1);
 	LOGDEB((stderr, "Parse:line: [%s] status %d\n", cline, int(status)));
 	if (!input.good()) {
 	    if (input.bad()) {
+                LOGDEB((stderr, "Parse: input.bad()\n"));
 		status = STATUS_ERROR;
 		return;
 	    }
-	    // Must be eof ?
-	    break;
+            LOGDEB((stderr, "Parse: eof\n"));
+	    // Must be eof ? But maybe we have a partial line which
+	    // must be processed. This happens if the last line before
+	    // eof ends with a backslash, or there is no final \n
+            eof = true;
 	}
 
-	int ll = strlen(cline);
-	while (ll > 0 && (cline[ll-1] == '\n' || cline[ll-1] == '\r')) {
-	    cline[ll-1] = 0;
-	    ll--;
-	}
+        {
+            int ll = strlen(cline);
+            while (ll > 0 && (cline[ll-1] == '\n' || cline[ll-1] == '\r')) {
+                cline[ll-1] = 0;
+                ll--;
+            }
+        }
 
 	if (appending)
 	    line += cline;
@@ -84,9 +93,11 @@ void ConfSimple::parseinput(istream &input)
 	    line = cline;
 
 	// Note that we trim whitespace before checking for backslash-eol
-	// This avoids invisible problems.
+	// This avoids invisible whitespace problems.
 	trimstring(line);
 	if (line.empty() || line.at(0) == '#') {
+            if (eof)
+                break;
 	    m_order.push_back(ConfLine(ConfLine::CFL_COMMENT, line));
 	    continue;
 	}
@@ -130,27 +141,29 @@ void ConfSimple::parseinput(istream &input)
 	    continue;
 	}
 	i_set(nm, val, submapkey, true);
+        if (eof)
+            break;
     }
 }
 
 
 ConfSimple::ConfSimple(int readonly, bool tildexp)
-    : dotildexpand(tildexp), m_data(0), m_holdWrites(false)
+    : dotildexpand(tildexp), m_holdWrites(false)
 {
     status = readonly ? STATUS_RO : STATUS_RW;
 }
 
-ConfSimple::ConfSimple(string *d, int readonly, bool tildexp)
-    : dotildexpand(tildexp), m_data(d), m_holdWrites(false)
+ConfSimple::ConfSimple(const string& d, int readonly, bool tildexp)
+    : dotildexpand(tildexp), m_holdWrites(false)
 {
     status = readonly ? STATUS_RO : STATUS_RW;
 
-    stringstream input(*d, ios::in);
+    stringstream input(d, ios::in);
     parseinput(input);
 }
 
 ConfSimple::ConfSimple(const char *fname, int readonly, bool tildexp)
-    : dotildexpand(tildexp), m_filename(fname), m_data(0), m_holdWrites(false)
+    : dotildexpand(tildexp), m_filename(fname), m_holdWrites(false)
 {
     status = readonly ? STATUS_RO : STATUS_RW;
 
@@ -185,7 +198,7 @@ ConfSimple::ConfSimple(const char *fname, int readonly, bool tildexp)
     parseinput(input);
 }
 
-ConfSimple::StatusCode ConfSimple::getStatus()
+ConfSimple::StatusCode ConfSimple::getStatus() const
 {
     switch (status) {
     case STATUS_RO: return STATUS_RO;
@@ -194,18 +207,18 @@ ConfSimple::StatusCode ConfSimple::getStatus()
     }
 }
 
-int ConfSimple::get(const string &nm, string &value, const string &sk)
+int ConfSimple::get(const string &nm, string &value, const string &sk) const
 {
     if (!ok())
 	return 0;
 
     // Find submap
-    map<string, map<string, string> >::iterator ss;
+    map<string, map<string, string> >::const_iterator ss;
     if ((ss = m_submaps.find(sk)) == m_submaps.end()) 
 	return 0;
 
     // Find named value
-    map<string, string>::iterator s;
+    map<string, string>::const_iterator s;
     if ((s = ss->second.find(nm)) == ss->second.end()) 
 	return 0;
     value = s->second;
@@ -404,7 +417,8 @@ ConfSimple::sortwalk(WalkerCode (*walker)(void *,const string&,const string&),
     return WALK_CONTINUE;
 }
 
-// Write to default output:
+// Write to default output. This currently only does something if output is
+// a file
 bool ConfSimple::write()
 {
     if (!ok())
@@ -416,11 +430,11 @@ bool ConfSimple::write()
 	if (!output.is_open())
 	    return 0;
 	return write(output);
-    } else if (m_data) {
-	ostringstream output(*m_data, ios::out | ios::trunc);
-	return write(output);
     } else {
-	// No backing store, no writing
+	// No backing store, no writing. Maybe one day we'll need it with
+        // some kind of output string. This can't be the original string which
+        // is currently readonly.
+	//ostringstream output(m_ostring, ios::out | ios::trunc);
 	return 1;
     }
 }
@@ -428,7 +442,7 @@ bool ConfSimple::write()
 // Write out the tree in configuration file format:
 // This does not check holdWrites, this is done by write(void), which
 // lets ie: listall work even when holdWrites is set
-bool ConfSimple::write(ostream& out)
+bool ConfSimple::write(ostream& out) const
 {
     if (!ok())
 	return false;
@@ -483,7 +497,7 @@ void ConfSimple::listall()
     write(std::cout);
 }
 
-list<string> ConfSimple::getNames(const string &sk)
+list<string> ConfSimple::getNames(const string &sk, const char *pattern)
 {
     std::list<string> mylist;
     if (!ok())
@@ -494,6 +508,8 @@ list<string> ConfSimple::getNames(const string &sk)
     }
     map<string, string>::const_iterator it;
     for (it = ss->second.begin();it != ss->second.end();it++) {
+        if (pattern && FNM_NOMATCH == fnmatch(pattern, it->first.c_str(), 0))
+            continue;
 	mylist.push_back(it->first);
     }
     mylist.sort();
@@ -513,11 +529,24 @@ list<string> ConfSimple::getSubKeys()
     return mylist;
 }
 
+bool ConfSimple::hasNameAnywhere(const string& nm)
+{
+    list<string>keys = getSubKeys();
+    for (list<string>::const_iterator it = keys.begin(); 
+         it != keys.end(); it++) {
+        string val;
+        if (get(nm, val, *it))
+            return true;
+    }
+    return false;
+}
+
 // //////////////////////////////////////////////////////////////////////////
 // ConfTree Methods: conftree interpret keys like a hierarchical file tree
 // //////////////////////////////////////////////////////////////////////////
 
 int ConfTree::get(const std::string &name, string &value, const string &sk)
+    const
 {
     if (sk.empty() || sk[0] != '/') {
 	//	LOGDEB((stderr, "ConfTree::get: looking in global space\n"));
@@ -552,7 +581,7 @@ int ConfTree::get(const std::string &name, string &value, const string &sk)
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
-
+#include <string.h>
 #include <sstream>
 #include <iostream>
 #include <list>
@@ -901,7 +930,7 @@ int main(int argc, char **argv)
 	if (argc != 0)
 	    Usage();
 	string s;
-	ConfSimple c(&s);
+	ConfSimple c(s);
 	memtest(c);
 	exit(0);
     } else if  ((op_flags & OPT_V)) {
@@ -992,10 +1021,16 @@ int main(int argc, char **argv)
 	//printf("WALK\n");conf->sortwalk(mywalker, 0);
 	printf("\nNAMES in global space:\n");
 	list<string> names = conf->getNames("");
-	for (list<string>::iterator it = names.begin();it!=names.end();
-	     it++) 
-	    printf("%s\n", (*it).c_str());
-
+	for (list<string>::iterator it = names.begin();
+             it!=names.end(); it++) 
+	    cout << *it << " ";
+        cout << endl;
+	printf("\nNAMES in global space matching t* \n");
+	names = conf->getNames("", "t*");
+	for (list<string>::iterator it = names.begin();
+             it!=names.end(); it++) 
+	    cout << *it << " ";
+        cout << endl;
     }
 }
 
