@@ -39,7 +39,7 @@ using namespace std;
 #endif /* NO_NAMESPACES */
 
 class FsTreeWalker::Internal {
-    Options options;
+    int options;
     stringstream reason;
     list<string> skippedNames;
     list<string> skippedPaths;
@@ -53,7 +53,7 @@ class FsTreeWalker::Internal {
     friend class FsTreeWalker;
 };
 
-FsTreeWalker::FsTreeWalker(Options opts)
+FsTreeWalker::FsTreeWalker(int opts)
 {
     data = new Internal;
     if (data) {
@@ -94,8 +94,6 @@ bool FsTreeWalker::addSkippedName(const string& pattern)
 bool FsTreeWalker::setSkippedNames(const list<string> &patterns)
 {
     data->skippedNames = patterns;
-    data->skippedNames.sort();
-    data->skippedNames.unique();
     return true;
 }
 bool FsTreeWalker::inSkippedNames(const string& name)
@@ -112,7 +110,7 @@ bool FsTreeWalker::inSkippedNames(const string& name)
 
 bool FsTreeWalker::addSkippedPath(const string& ipath)
 {
-    string path = path_canon(ipath);
+    string path = (data->options & FtwNoCanon) ? ipath : path_canon(ipath);
     if (find(data->skippedPaths.begin(), 
 	     data->skippedPaths.end(), path) == data->skippedPaths.end())
 	data->skippedPaths.push_back(path);
@@ -123,18 +121,33 @@ bool FsTreeWalker::setSkippedPaths(const list<string> &paths)
     data->skippedPaths = paths;
     for (list<string>::iterator it = data->skippedPaths.begin();
 	 it != data->skippedPaths.end(); it++)
-	*it = path_canon(*it);
-    data->skippedPaths.sort();
-    data->skippedPaths.unique();
+        if (!(data->options & FtwNoCanon))
+            *it = path_canon(*it);
     return true;
 }
-bool FsTreeWalker::inSkippedPaths(const string& path)
+bool FsTreeWalker::inSkippedPaths(const string& path, bool ckparents)
 {
+    int fnmflags = FNM_PATHNAME;
+#ifdef FNM_LEADING_DIR
+    if (ckparents)
+        fnmflags |= FNM_LEADING_DIR;
+#endif
     list<string>::const_iterator it;
     for (it = data->skippedPaths.begin(); 
 	 it != data->skippedPaths.end(); it++) {
-	if (fnmatch(it->c_str(), path.c_str(), FNM_PATHNAME) == 0) 
-	    return true;
+#ifndef FNM_LEADING_DIR
+        if (ckparents) {
+            string mpath = path;
+            while (mpath.length() > 2) {
+                if (fnmatch(it->c_str(), mpath.c_str(), fnmflags) == 0) 
+                    return true;
+                mpath = path_getfather(mpath);
+            }
+        } else 
+#endif /* FNM_LEADING_DIR */
+        if (fnmatch(it->c_str(), path.c_str(), fnmflags) == 0) {
+            return true;
+        }
     }
     return false;
 }
@@ -142,7 +155,7 @@ bool FsTreeWalker::inSkippedPaths(const string& path)
 FsTreeWalker::Status FsTreeWalker::walk(const string& _top, 
 					FsTreeWalkerCB& cb)
 {
-    string top = path_canon(_top);
+    string top = (data->options & FtwNoCanon) ? _top : path_canon(_top);
 
     struct stat st;
     int statret;
@@ -213,7 +226,12 @@ FsTreeWalker::Status FsTreeWalker::iwalk(const string &top,
 		continue;
 	    }
 	    if (!data->skippedPaths.empty()) {
-		if (inSkippedPaths(fn))
+                // We do not check the ancestors. This means that you can have
+                // a topdirs member under a skippedPath, to index a portion of
+                // an ignored area. This is the way it had always worked, but
+                // this was broken by 1.13.00 and the systematic use of 
+                // FNM_LEADING_DIR
+		if (inSkippedPaths(fn, false))
 		    goto skip;
 	    }
 
@@ -225,9 +243,12 @@ FsTreeWalker::Status FsTreeWalker::iwalk(const string &top,
 		}
 		if (status & (FtwStop|FtwError))
 		    goto out;
-		if ((status = cb.processone(top, &st, FtwDirReturn)) 
-		    & (FtwStop|FtwError))
-		    goto out;
+
+                // No DirReturn call when not recursing
+                if (!(data->options & FtwNoRecurse)) 
+                    if ((status = cb.processone(top, &st, FtwDirReturn)) 
+                        & (FtwStop|FtwError))
+                        goto out;
 	    } else if (S_ISREG(st.st_mode)) {
 		if ((status = cb.processone(fn, &st, FtwRegular)) & 
 		    (FtwStop|FtwError)) {
@@ -256,14 +277,24 @@ FsTreeWalker::Status FsTreeWalker::iwalk(const string &top,
 
 using namespace std;
 
+static int     op_flags;
+#define OPT_MOINS 0x1
+#define OPT_p	  0x2 
+#define OPT_P	  0x4 
+#define OPT_r     0x8
+#define OPT_c     0x10
+
 class myCB : public FsTreeWalkerCB {
  public:
     FsTreeWalker::Status processone(const string &path, 
-				 const struct stat *st,
-				 FsTreeWalker::CbFlag flg)
+                                    const struct stat *st,
+                                    FsTreeWalker::CbFlag flg)
     {
 	if (flg == FsTreeWalker::FtwDirEnter) {
-	    cout << "[Entering " << path << "]" << endl;
+            if (op_flags & OPT_r) 
+                cout << path << endl;
+            else
+                cout << "[Entering " << path << "]" << endl;
 	} else if (flg == FsTreeWalker::FtwDirReturn) {
 	    cout << "[Returning to " << path << "]" << endl;
 	} else if (flg == FsTreeWalker::FtwRegular) {
@@ -276,7 +307,9 @@ class myCB : public FsTreeWalkerCB {
 static const char *thisprog;
 
 static char usage [] =
-"trfstreewalk [-p pattern] [-P ignpath] topdir\n\n"
+"trfstreewalk [-p pattern] [-P ignpath] [-r] [-c] topdir\n"
+" -r : norecurse\n"
+" -c : no path canonification\n"
 ;
 static void
 Usage(void)
@@ -285,11 +318,6 @@ Usage(void)
     exit(1);
 }
 
-static int     op_flags;
-#define OPT_MOINS 0x1
-#define OPT_p	  0x2 
-#define OPT_P	  0x4 
-
 int main(int argc, const char **argv)
 {
     list<string> patterns;
@@ -304,6 +332,8 @@ int main(int argc, const char **argv)
       Usage();
     while (**argv)
       switch (*(*argv)++) {
+      case 'r':	op_flags |= OPT_r; break;
+      case 'c':	op_flags |= OPT_c; break;
       case 'p':	op_flags |= OPT_p; if (argc < 2)  Usage();
 	  patterns.push_back(*(++argv));
 	  argc--; 
@@ -321,7 +351,12 @@ int main(int argc, const char **argv)
     Usage();
   string topdir = *argv++;argc--;
 
-  FsTreeWalker walker;
+  int opt = 0;
+  if (op_flags & OPT_r)
+      opt |= FsTreeWalker::FtwNoRecurse;
+  if (op_flags & OPT_c)
+      opt |= FsTreeWalker::FtwNoCanon;
+  FsTreeWalker walker(opt);
   walker.setSkippedNames(patterns);
   walker.setSkippedPaths(paths);
   myCB cb;
