@@ -39,12 +39,16 @@ struct stat;
  */
 class FsTreeWalker {
  public:
+    // Flags for call to processone(). FtwDirEnter is used when
+    // entering a directory. FtwDirReturn is used when returning to it
+    // after processing a subdirectory.
     enum CbFlag {FtwRegular, FtwDirEnter, FtwDirReturn};
     enum Status {FtwOk=0, FtwError=1, FtwStop=2, 
 		 FtwStatAll = FtwError|FtwStop};
-    enum Options {FtwOptNone = 0, FtwNoRecurse = 1, FtwFollow = 2};
+    enum Options {FtwOptNone = 0, FtwNoRecurse = 1, FtwFollow = 2,
+                  FtwNoCanon = 4};
 
-    FsTreeWalker(Options opts = FtwOptNone);
+    FsTreeWalker(int opts = FtwOptNone);
     ~FsTreeWalker();
 
     void setOpts(Options opts);
@@ -75,8 +79,11 @@ class FsTreeWalker {
     /** Set the ignored paths list */
     bool setSkippedPaths(const list<string> &pathlist);
 
-    bool inSkippedPaths(const string& path);
+    /** Test if path/name should be skipped. This can be used independantly of
+      * an actual tree walk */
+    bool inSkippedPaths(const string& path, bool ckparents = true);
     bool inSkippedNames(const string& name);
+
  private:
     Status iwalk(const string &dir, struct stat *stp, FsTreeWalkerCB& cb);
     class Internal;
