@@ -182,71 +182,71 @@ template <class T> bool stringToStrings(const string &s, T &tokens)
     states state = SPACE;
     for (unsigned int i = 0; i < s.length(); i++) {
 	switch (s[i]) {
-	    case '"': 
+        case '"': 
 	    switch(state) {
-	      case SPACE: 
+            case SPACE: 
 		state=INQUOTE; continue;
-	      case TOKEN: 
+            case TOKEN: 
 	        current += '"';
 		continue;
-	      case INQUOTE: 
-		tokens.push_back(current);
+            case INQUOTE: 
+                tokens.insert(tokens.end(), current);
 		current.clear();
 		state = SPACE;
 		continue;
-	      case ESCAPE:
+            case ESCAPE:
 	        current += '"';
 	        state = INQUOTE;
-	      continue;
+                continue;
 	    }
 	    break;
-	    case '\\': 
+        case '\\': 
 	    switch(state) {
-	      case SPACE: 
-	      case TOKEN: 
-		  current += '\\';
-		  state=TOKEN; 
-		  continue;
-	      case INQUOTE: 
-		  state = ESCAPE;
-		  continue;
-	      case ESCAPE:
-		  current += '\\';
-		  state = INQUOTE;
-		  continue;
+            case SPACE: 
+            case TOKEN: 
+                current += '\\';
+                state=TOKEN; 
+                continue;
+            case INQUOTE: 
+                state = ESCAPE;
+                continue;
+            case ESCAPE:
+                current += '\\';
+                state = INQUOTE;
+                continue;
 	    }
 	    break;
 
-	    case ' ': 
-	    case '\t': 
-	    case '\n': 
-	    case '\r': 
+        case ' ': 
+        case '\t': 
+        case '\n': 
+        case '\r': 
 	    switch(state) {
-	      case SPACE: 
-		  continue;
-	      case TOKEN: 
-		tokens.push_back(current);
+            case SPACE: 
+                continue;
+            case TOKEN: 
+		tokens.insert(tokens.end(), current);
 		current.clear();
 		state = SPACE;
 		continue;
-	      case INQUOTE: 
-	      case ESCAPE:
-		  current += s[i];
-		  continue;
+            case INQUOTE: 
+            case ESCAPE:
+                current += s[i];
+                continue;
 	    }
 	    break;
 
-	    default:
+        default:
 	    switch(state) {
-	      case ESCAPE:
-		  state = INQUOTE;
-		  break;
-	      case SPACE: 
-		  state = TOKEN;
-		  break;
-	      case TOKEN: 
-	      case INQUOTE: 
-		  break;
+            case ESCAPE:
+                state = INQUOTE;
+                break;
+            case SPACE: 
+                state = TOKEN;
+                break;
+            case TOKEN: 
+            case INQUOTE: 
+                break;
 	    }
 	    current += s[i];
 	}
@@ -255,7 +255,7 @@ template <class T> bool stringToStrings(const string &s, T &tokens)
     case SPACE: 
 	break;
     case TOKEN: 
-	tokens.push_back(current);
+	tokens.insert(tokens.end(), current);
 	break;
     case INQUOTE: 
     case ESCAPE:
@@ -271,6 +271,10 @@ bool stringToStrings(const string &s, vector<string> &tokens)
 {
     return stringToStrings<vector<string> >(s, tokens);
 }
+bool stringToStrings(const string &s, set<string> &tokens)
+{
+    return stringToStrings<set<string> >(s, tokens);
+}
 
 template <class T> void stringsToString(const T &tokens, string &s) 
 {
@@ -492,7 +496,8 @@ bool pcSubst(const string& in, string& out, map<char, string>& subs)
 	    if ((tr = subs.find(*it)) != subs.end()) {
 		out += tr->second;
 	    } else {
-		out += *it;
+		// We used to do "out += *it;" here but this does not make
+                // sense
 	    }
 	} else {
 	    out += *it;
@@ -501,6 +506,52 @@ bool pcSubst(const string& in, string& out, map<char, string>& subs)
     return true;
 }
 
+bool pcSubst(const string& in, string& out, map<string, string>& subs)
+{
+    out.erase();
+    string::size_type i;
+    for (i = 0; i < in.size(); i++) {
+	if (in[i] == '%') {
+	    if (++i == in.size()) {
+		out += '%';
+		break;
+	    }
+	    if (in[i] == '%') {
+		out += '%';
+		continue;
+	    }
+            string key = "";
+            if (in[i] == '(') {
+                if (++i == in.size()) {
+                    out += string("%(");
+                    break;
+                }
+                string::size_type j = in.find_first_of(")", i);
+                if (j == string::npos) {
+                    // ??concatenate remaining part and stop
+                    out += in.substr(i-2);
+                    break;
+                }
+                key = in.substr(i, j-i);
+                i = j;
+            } else {
+                key = in[i];
+            }
+	    map<string,string>::iterator tr;
+	    if ((tr = subs.find(key)) != subs.end()) {
+		out += tr->second;
+	    } else {
+                // Substitute to nothing, that's the reasonable thing to do
+                // instead of keeping the %(key)
+                // out += key.size()==1? key : string("(") + key + string(")");
+	    }
+	} else {
+	    out += in[i];
+	}
+    }
+    return true;
+}
+
 // Convert byte count into unit (KB/MB...) appropriate for display
 string displayableBytes(long size)
 {
@@ -712,11 +763,34 @@ int main(int argc, char **argv)
 	utf8truncate(testit, sz);
 	cout << testit << endl;
     }
-#elif 1
+#elif 0
     std::string testit("ligne\ndeuxieme ligne\r3eme ligne\r\n");
     cout << "[" << neutchars(testit, "\r\n") << "]" << endl;
     string i, o;
     cout << "neutchars(null) is [" << neutchars(i, "\r\n") << "]" << endl;
+#elif 1
+    map<string, string> substs;
+    substs["a"] = "A_SUBST";
+    substs["title"] = "TITLE_SUBST";
+    string in = "a: %a title: %(title) pcpc: %% %";
+    string out;
+    pcSubst(in, out, substs);
+    cout << in << " => " << out << endl;
+
+    in = "unfinished: %(unfinished";
+    pcSubst(in, out, substs);
+    cout << in << " => " << out << endl;
+    in = "unfinished: %(";
+    pcSubst(in, out, substs);
+    cout << in << " => " << out << endl;
+    in = "empty: %()";
+    pcSubst(in, out, substs);
+    cout << in << " => " << out << endl;
+    substs.clear();
+    in = "a: %a title: %(title) pcpc: %% %";
+    pcSubst(in, out, substs);
+    cout << "After map clear: " << in << " => " << out << endl;
+
 #endif
 
 }
