@@ -17,6 +17,8 @@
 #ifndef _SMALLUT_H_INCLUDED_
 #define _SMALLUT_H_INCLUDED_
 /* @(#$Id: smallut.h,v 1.33 2008-12-05 07:38:07 dockes Exp $  (C) 2004 J.F.Dockes */
+#include <stdlib.h>
+
 #include <string>
 #include <list>
 #include <vector>
@@ -54,6 +56,7 @@ extern bool samecharset(const string &cs1, const string &cs2);
  */
 extern bool stringToStrings(const string &s, list<string> &tokens);
 extern bool stringToStrings(const string &s, vector<string> &tokens);
+extern bool stringToStrings(const string &s, set<string> &tokens);
 
 /**
  * Inverse operation:
@@ -102,6 +105,8 @@ string breakIntoLines(const string& in, unsigned int ll = 100,
 		      unsigned int maxlines= 50);
 /** Small utility to substitute printf-like percents cmds in a string */
 bool pcSubst(const string& in, string& out, map<char, string>& subs);
+/** Substitute printf-like percents and also %(key) */
+bool pcSubst(const string& in, string& out, map<string, string>& subs);
 
 class Chrono {
  public:
@@ -125,6 +130,25 @@ class Chrono {
   long 	m_nsecs; 
 };
 
+class TempBuf {
+public:
+    TempBuf() 
+        : m_buf(0)
+    {}
+    TempBuf(int n)
+    {
+        m_buf = (char *)malloc(n);
+    }
+    ~TempBuf()
+    { 
+        if (m_buf)
+            free(m_buf);
+    }
+    char *setsize(int n) { return (m_buf = (char *)realloc(m_buf, n)); }
+    char *buf() {return m_buf;}
+    char *m_buf;
+};
+
 #ifndef deleteZ
 #define deleteZ(X) {delete X;X = 0;}
 #endif
